class MolType:
    """Base class describing a molecular type (AA or CG)"""        
    def __init__(self,Name,type='',db=''):        
        from sys import exit
        if (type==''): # File type not stated - detect from the name
            type=Name.strip().split('.')[-1]            
            Name='.'.join(Name.strip().split('.')[0:-1])
        self.Name=Name #Name of the type
        self._filetype=type # Type of the file describing moltype
        self._filedb=db     # Directory where all moletype-files are stored
        self._initialized_=False
        self.NAtom=0      # Number of atoms in the moltype
        self.Atoms=[]       # List of atoms in the moltype
        self.Bonds=[]       # List of bonds in the molecular type
        if self._filetype=='mmol':
            filename="./"+self._filedb+"/"+self.Name+".mmol"
            self._read_moltype_from_file_mmol(filename)
        elif (self._filetype=='mcm'):
            filename="./"+self._filedb+"/"+self.Name+".mcm"
            self._read_moltype_from_file_mcm(filename)
        elif (self._filetype=='dump'):
#            filename="./"+self._filedb+"/"+self.Name+".dump"
#            self._read_moltype_from_file_tprdump(filename)
            pass
        else:
            print 'Unknown type of the file:',self._filetype; exit(2)
    def _read_moltype_from_file_mmol(self,filename):
        import Atom
        from sys import exit
        print 'read_moltype_from_file_mmol:',filename
        try: 
            ifile=open(filename,'r')
            ll=ifile.readlines()    
        except: 
            print 'Unable to open and read file:',filename; exit(2)
    # cut away all lines starting with # or !
        ll = map(lambda s: s.strip(), ll)
        ll = map(lambda s: s.lstrip(), ll)
        ll = filter(lambda s: not s.startswith(('#','!')),ll)        
        #read number of atoms in the molecular type
        try:
            self.NAtom=int(ll[0].strip())
        except: 
            print 'Unable to read number of atoms in file:',filename,'got:',ll[0].strip(); exit(2)
        name=""
        x=0.0
        y=0.0
        z=0.0
        mass=0.0
        charge=0.0
        try:
            for i in range(0,self.NAtom):
                lineatom=ll[i+1].split()
                name=lineatom[0].strip()                
                x=float(lineatom[1])
                y=float(lineatom[2])
                z=float(lineatom[3])
                mass=float(lineatom[4])
                charge=float(lineatom[5])
                a=Atom.Atom(name,x,y,z,mass,charge)
                print "Atom added: ;%s; %5.3f %5.3f %5.3f %5.2f %5.3f" %(name, x,y,z,mass,charge)
                self.Atoms.append(a)
                self._initialized_=True
            print ""
        except:            
            print 'Unable to read atom records in file:',filename,' line:',(i+2); exit(2)
        

    def _read_moltype_from_file_mcm(self,mcmfile):
        from sys import exit
        # REad lines from mcmfile, and clean them from lineend and comment lines
        import Atom
        mcm=open(mcmfile,'r')
        lines = mcm.readlines()
        lines = map(lambda s: s.strip(), lines)
        lines = map(lambda s: s.lstrip(), lines)
        lines = filter(lambda s: (s[0]!='!')&(s[0]!='#'),lines)
        mcm.close()

        # Read the atom-list part of the mcmfile
        NAtoms=int(lines[0])
        Atoms=[]
        for l in lines[1:NAtoms+1]:
            atom_name=l.split()[0]
            atom_coord=map(float,l.split()[1:4])
            atom_mass=float(l.split()[4])
            atom_charge=float(l.split()[5])
            atom_TypeID=int(l.split()[6])
            atom_Type=l.split()[7]
            atom=Atom.Atom(atom_name, atom_coord[0], atom_coord[1], atom_coord[2], atom_mass, atom_charge)
            atom.Type=atom_Type
            atom.TypeID=atom_TypeID
            Atoms.append(atom)
        for i in xrange(0,len(Atoms)):
            Atoms[i].ID=i+1        
        # Read pairwise-bonds
        NBonds=int(lines[NAtoms+1])
        i=NAtoms+1
        Bonds=[]
        for iBond in xrange(1,NBonds+1):
            i+=1
            N_atomgroups=int(lines[i])
            atomgroups=[]
            for grp in xrange(0,N_atomgroups):
                i+=1
                atomgroups.append(map(int,lines[i].split()[0:2]))
            if (len(atomgroups)!=N_atomgroups):
                print "Error in mcm-file {0}. Bond {1}. Number of atom groups stated for the bond {2}, differs from the actual number of provided groups {3}".format(mcmfile,iBond, N_atomgroups, atomgroups)
                exit()
            Bonds.append(Atom.Bond(iBond,atomgroups,'B',self))

        # Read angle-bending bonds
        # Check the order in atom-triplets
        if 'Order=1-2-3' in lines[i+1]:
            FlagNewOrder=True
            lines[i+1]=lines[i+1].replace('Order=1-2-3','')
        else:
            FlagNewOrder=False
        NAngles=int(lines[i+1])
        i+=1
        Angles=[]
        for iBond in xrange(NBonds+1,NBonds+NAngles+1):
            i+=1
            N_atomgroups=int(lines[i])
            atomgroups=[]
            for grp in xrange(0,N_atomgroups):
                i+=1
                if FlagNewOrder:
                    atomgroups.append(map(int,lines[i].split()[0:3]))
                elif not FlagNewOrder:
                    atomgroups.append([int(lines[i].split()[j]) for j in [0,2,1]])
            if (len(atomgroups)!=N_atomgroups):
                print "Error in mcm-file {0}. Bond {1}. Number of atom groups stated for the angle-bond {2}, differs from the actual number of provided groups {3}".format(mcmfile,iBond, N_atomgroups, atomgroups)
                exit()
            Angles.append(Atom.Bond(iBond,atomgroups,'A',self))
                
        # Creating a MolType object
        self.Atoms=Atoms
        self.Bonds=Bonds
        self.NAtom=NAtoms
        self.ABonds=Angles
        self._initialized_=True
    
    def _read_moltype_from_file_tprdump(self,tprfile):    
        import re

        ifile=open('out','r')
        lines=ifile.readlines()
        ifile.close()
        MTlines=[line.strip() for line in lines if re.search('moltype +=',line)!=None]
        MTNames=[re.search('".*"',line).group().replace('"','') for line in MTlines]
        
        NMollines=[line.strip() for line in lines if re.search('#molecules +=',line)!=None]
        NMols=[int(line.split('=')[1]) for line in NMollines]
        
        NAtomlines=[line.strip() for line in lines if re.search('#atoms_mol +=',line)!=None]
        NAtoms=[int(line.split('=')[1]) for line in NAtomlines]
        
        lBegMT=[i for iMT, MT in enumerate(MTNames) for i,l in enumerate(lines) if 'moltype ({0:<d})'.format(iMT) in l]
        lBegMT.append(len(lines)-1)
        
        Rlines=[line.strip() for line in lines if re.search(' +x\[ *[0-9]*\]=\{.*\}',line)!=None]
        R=[[float(rec) for rec in line.replace('}','').split(']={')[1].split(',')] for line in Rlines ]
        rBeg=0
        for iMT, MT in enumerate(MTNames):
            ll=lines[lBegMT[iMT]:lBegMT[iMT+1]]
            ANamelines=[line.strip() for line in ll if re.search('atom\[[0-9]*\]=\{name=.*\}',line)!=None]
            ANames=[re.search('".*"',line).group().replace('"','') for line in ANamelines]
            AProplines=[line.strip() for line in ll if re.search('atom\[ *[0-9]*\]=\{type=.*\}',line)!=None]
            AProps=[rec.split('=')[1].strip()[i] for line in AProplines for rec in line.split(']={')[1].split(',') for i in [0,3,4]]
            R=Rlines[rBeg:rBeg+NAtoms[iMT]]
            rBeg+=NMols[iMT]*NAtoms[iMT]
        
    def WriteMCM(self):            
        import Atom
        try:
            ofile=open(self.Name+'.mcm','w')
        except:
             print "Error:Unable to create mcm output file:",self.Name; exit(7)
        ofile.write(str(self.NAtom)+'\n')
        # Caclulate CG-atom type for each atom in list

        #Writing atoms description line by line ordered by atomtype
        
        
        for atom in self.Atoms:
#            i=-1
#            for iat in xrange(0,len(self.AType)):
#                if atom in self.AType[iat].Atoms:
#                    i=iat
#            if i==-1:
#                print "Atom type of atom:",atom.Name,' not found'; exit(7)
#            atypes.append(i)
#            line=atom.Name.ljust(5)+' '+str(atom.X)+' '+str(atom.Y)+' '+str(atom.Z) \
#            +' '+str(atom.Mass)+' '+str(atom.Charge)+' '+str(i+1)+' '+self.AType[i].Name+'\n'
            line='{0:<5s} {1} {2} {3} {4} {5} {6} {7}\n'.format(atom.Name, atom.X, atom.Y, atom.Z,
                atom.Mass, atom.Charge, atom.TypeID, atom.Type)
            ofile.write(line)                    
        

        #Writing bonds
        ofile.write('{0}\n'.format(len([b for b in self.Bonds if b.Type=='B'])))
        for b in [bb for bb in self.Bonds if bb.Type=='B']:
            ofile.write(str(len(b.AtomGroups))+'\n')
            for ap in b.AtomGroups:
                if isinstance(ap[0],Atom.Atom):
                    ofile.write('{0} {1}\n'.format(ap[0].Number, ap[1].Number))
                else:
                    ofile.write('{0} {1}\n'.format(ap[0], ap[1]))

        #Writing Angle-Bonds
        ofile.write('{0}  Order=1-2-3 \n'.format(len([b for b in self.Bonds if b.Type=='A'])))
        for b in [bb for bb in self.Bonds if bb.Type=='A']:
            ofile.write(str(len(b.AtomGroups))+'\n')
            for ap in b.AtomGroups:
                if isinstance(ap[0],Atom.Atom):
                    ofile.write('{0} {1} {2}\n'.format(ap[0].Number, ap[1].Number, ap[2].Number))
                else:
                   ofile.write('{0} {1} {2}\n'.format(ap[0], ap[1], ap[2]))
        ofile.close()
    def __str__(self):
        return self.Name