module iomod   
    use precision
    use strings
    use utilsmod
    use dynarray
    use mcmio
    use rdfio
    use potio
 
    implicit none

    integer, private:: iPrint, stdout, ID, rank, psize
   
    interface ReadParameter  ! Generic operator for reading a parameter from input file
        module procedure ReadParameterInt
        module procedure ReadParameterInt8
        module procedure ReadParameterReal
        module procedure ReadParameterString
        module procedure ReadParameterLogical        
    end interface    
    contains
#include "iomod_readinput.f90"
subroutine ReadAll(&
    ! ReadInput    
                   NMType,&                   
                   NameMType, & 
                   NMolMType, &
                   NA,&
                   LMoveMType,&
                   LReadRDF, LReadPot,&
                   NMCSteps, NMCStepsEquil, MCStepAtomMType, MCTransStepMType, MCRotStepMType, ITRANS, IROT,&
                   RECUT, NRS, POTCUT,AF, FQ, IOUT, &
                   IAV, REGP, DPOTM, LCRD, LCRDPASS, RTM, IREPT, LIMC, IPOTCORRCHECK, LPotCorrCheck,PotCorrCheckLevel, &
                   LEV, BaseOutFileName, FSTART, NFStart,FCRD, LXMOL, ITR, &
                   NMCAutoAdjust, MCStepAtomARMType,MCTransStepARMType, MCRotStepARMType,&
! SetParameters-related
                   VOL, BOX, FRT, TEMP, FKJM, FPR, COULF, EPS, iAverage, &
!  ReadMCM-related                   
                   NSiteMType, NBondMType, SumMassMType,&
                   NSite, NameSite, MTypeSite, RSite, MassSite,ChargeSite,  &
                   NSType,MTypeSType, NameSType,&    
                   NBond, NBondB, NBondA, Bond_iSjS, iSiteInBond, jSiteInBond, kSiteInBond, NSiteGroupsInBond,&
                   BTypeBond,MTypeBond,&
! Set References
                   NMol,MTypeMol,XvecCOMMol,&
                   NAtom, MolAtom, STypeAtom, SiteAtom, ChargeAtom,&
                   NAtomChrgatom, AtomChrgatom, &
                   AdrAtomMol, AdrMolMType, AdrSiteMType, AdrAtomMType, AdrBondMType,&
                   NPairsTotalBond, NMIN, Svec, Wvec,&                   
! Setreferences-SetAngleBonds                   
                   SitesInGroupBondMType, SideSiteABond, NABondSite, SiteABondMType,EdgeSiteABond_iSjS,&
!ReadRDF                   
                   FilRDF,&
                   StateNB_iSjS, StateB_iBond,&
!                   NPointsGlobNB, NPointsGlobB, NPointsGlobTotal,&
                   IndexGlobNB_riSjS, IndexGlobB_riBond,&
                   RMaxRDFNB,RMinRDFNB, RDFref_NB_riSjS, RDFref_B_riBond,&
!ReadPot                   
                   FilPot,&
                   PotNB_riSjS, PotB_riBond,&
                   NPointsPotB, RMaxPotB, RMinPotB,&
! Initialize                    
                   NactNB,&
                   IND,INDI,&
                   CORP,CORS,npairsrdf,AdrGlobIndexAtomij,&
                   RCUT,RDFINC,&
    stdout_,iPrint_, rank, ID, psize)   

    integer NMType
    character(mol_name_len), allocatable:: NameMType(:)
    integer, allocatable:: NMolMType(:)
    integer NA
    logical, allocatable:: LMoveMType(:)
    logical LReadRDF, LReadPot
    integer stdout_, iPrint_
    real(8) PotCorrCheckLevel
    real(8) VOL, FRT,  FKJM, FPR, COULF
    real(8), allocatable:: SHELV(:), RAS(:)
    
    integer, allocatable:: NSiteMType(:),NBondMType(:)    
    real(8), allocatable :: SumMassMType(:)

    integer NSite, NSType
    integer, allocatable:: IndexSTypeSite(:),MTypeSite(:), MTypeSType(:)!, NSiteSType(:)
    character(atom_name_len), allocatable:: NameSite(:), NameSTypeSite(:), NameSType(:)
    integer, allocatable:: Bond_iSjS(:,:), iSiteInBond(:,:), jSiteInBond(:,:), kSiteInBond(:,:), NSiteGroupsInBond(:),&
    NPairsTotalBond(:), BTypeBond(:),MTypeBond(:), NMIN(:)
    integer NBond,NBondB, NBondA, NActNB
    real(8), allocatable:: RSite(:,:)                       
    real(8), allocatable:: MassSite(:), ChargeSite(:)            
!---------------------------------------------------------------------------------            
    character(filename_text) FilRDF, FilPot, iFilename,BaseOutFileName, FSTART, FCRD
    integer(4), intent(out):: NFStart
!---SetReferences--------------------------------------------------------------
    integer NMol, NAtom, NAtomChrgatom, NPointsNBGpot, iAverage
    integer, allocatable :: AdrAtomMol(:), AdrMolMType(:), AdrSiteMType(:), AdrAtomMType(:),AdrBondMType(:), &
    MTypeMol(:), &
    MolAtom(:), STypeAtom(:), SiteAtom(:), &
    AtomChrgatom(:), &
    indexGPotNB_riSjS(:,:,:), IND(:,:,:), INDI(:,:), &
    StateNB_iSjS(:,:), StateB_iBond(:), &
    NPointsRDFB(:),NPointsPotB(:),&
    iRGPot(:), iSTypeNBGpot(:), jSTypeNBGpot(:)
    real(8), allocatable :: XvecCOMMol(:,:),ChargeAtom(:),Svec(:,:),Wvec(:,:)
    
    integer(4), allocatable :: SitesInGroupBondMType(:,:,:,:),& ! I-th triplet of sites involved in covalent-angle bond of type IBB-NBondB (numbering excluding pair bonds) in molecular type ITYP
        SideSiteABond(:,:,:),& ! I-th dublet of sites involved in same covalent angle bond as site isite. Given site are sides of angle and isite is its edge
        NABondSite(:),& !  number of covalent angle bonds which site isite involved in
        SiteABondMType(:,:),& !  List of angle bond types in molecular type NMType. Last cell (NBondA+1,NMType) keeps number of bond types, and first cells in a row keep numbers ov these types.
        EdgeSiteABond_iSjS(:,:)
    
!---ReadRDF---------------------------------------------------------
    real(8), allocatable::RDFref_NB_riSjS(:,:,:), RDFref_B_riBond(:,:), rRDFB_riBond(:,:)!,RDFA_iBond(:)    
    real(8), allocatable::PotNB_riSjS(:,:,:), PotB_riBond(:,:), rPotB_riBond(:,:)!,RDFA_iBond(:)
    real(8), allocatable::RMaxRDFB(:),RMinRDFB(:),RMaxPotB(:),RMinPotB(:)
    integer NPointsRDFNB, NPointsPotNB
    real(8) RMaxRDFNB, RMinRDFNB, RMaxPotNB, RMinPotNB
!----Initiate-----    
    integer, allocatable:: IndexGlobNB_riSjS(:,:,:), IndexGlobB_riBond(:,:) 
    integer(2), allocatable:: AdrGlobIndexAtomij(:,:)
!    integer NPointsGlobNB, NPointsGlobB, NPointsGlobTotal
    integer(8), allocatable:: CORS(:),CORP(:,:),npairsrdf(:,:)
    real(8)  RCUT, RDFINC
!----ReadInput----------------    
    integer(4)  ITRANS, IROT, NRS, IAV, IREPT, IPOTCORRCHECK, ITR, IOUT
    integer(4), intent(out):: NMCAutoAdjust
    real(8) BOX(3), EPS, TEMP, RECUT, POTCUT, AF, FQ,&
        REGP, DPOTM, RTM
    real(8),allocatable, intent(out) :: MCStepAtomMType(:),MCTransStepMType(:), MCRotStepMType(:),&
                                        MCStepAtomARMType(:),MCTransStepARMType(:), MCRotStepARMType(:)
    integer(8) NMCSteps, NMCStepsEquil            
    logical LIMC, LZeroPot, LCRD, LCRDPASS, LEV, LXMOL, LPotCorrCheck
      
    character(filename_text) oFilename    
    integer i,j,k

 ! ------PrepareReadInput()
    integer(4) rank, ID, psize
    character(input_text), allocatable :: text(:)

    call PrepareReadInput(rank, ID, psize, iFilename, BaseOutFileName, text, iPrint_)
    stdout_=stdout
    call ReadInput(iFilename,&
            NMType,NameMType,NMolMType,LMoveMType,&
            BOX,&
            EPS,TEMP,&
            NMCSteps, NMCStepsEquil, MCStepAtomMType, MCTransStepMType, MCRotStepMType, ITRANS, IROT,&
            RECUT, NRS, POTCUT,AF, FQ, IOUT, &
            IAV, REGP, DPOTM, LCRD, LCRDPASS, RTM, IREPT, LIMC, &
            IPOTCORRCHECK, LPotCorrCheck,PotCorrCheckLevel, &
            LEV,&
            FilRDF, FilPot,LReadRDF, LReadPOT, BaseOutFileName, FSTART,NFStart,FCRD, &
            LXMOL, ITR,&
            NMCAutoAdjust,MCStepAtomARMType,MCTransStepARMType, MCRotStepARMType)
    write(stdout,*) ''            
    write(stdout,*) '-----------------------------------------------------------------------'    
    write(stdout,*) ''
    write(stdout,*) 'Reading mcm-files'
    write(stdout,*) ''
    call ReadMCM(NMType, NameMType,&
        NSiteMType, NBondMType, SumMassMType,&
        NSite, NameSite, IndexSTypeSite, MTypeSite, RSite, MassSite,ChargeSite,  &
        NSType,MTypeSType, NameSType,&    
        NBond, NBondB, NBondA, Bond_iSjS, iSiteInBond, jSiteInBond, kSiteInBond, NSiteGroupsInBond,BTypeBond,MTypeBond)
    write(stdout,*) ''
    write(stdout,*) '-----------------------------------------------------------------------'    
    write(stdout,*) ''
    write(stdout,*) 'Setting reference data structures:'    
    call SetReferences(NA, NMType, NMolMType, NSiteMType, NBondMType, &
        NMol, MTypeMol, XvecCOMMol, &
        NSType, NSite, MTypeSite, IndexSTypeSite, ChargeSite, &
        NAtom, MolAtom, STypeAtom, SiteAtom, ChargeAtom, &
        NAtomChrgatom, AtomChrgatom, &
        AdrAtomMol, AdrMolMType, AdrSiteMType, AdrAtomMType, AdrBondMType,&
        NBond, NBondB, NBondA, StateNB_iSjS, StateB_iBond,  NPairsTotalBond, NMIN, NSiteGroupsInBond, &
        RDFref_NB_riSjS, PotNB_riSjS,& 
        NPointsRDFB, RMaxRDFB, RMinRDFB, &
        NPointsPotB, RMaxPotB, RMinPotB, &
        ! Setreferences-SetAngleBonds                   
        SitesInGroupBondMType, SideSiteABond, NABondSite, SiteABondMType,EdgeSiteABond_iSjS,&
        BTypeBond,iSiteInBond, jSiteInBond, kSiteInBond,&
        Svec,Wvec,&
        LMoveMType, IRot, ITrans)
    write(stdout,*) 'Reference data structures set successfully'    
    write(stdout,*) ''    
    write(stdout,*) '-----------------------------------------------------------------------'        
    if (LReadRDF) then                 
        write(stdout,*) ''    
        write(stdout,'(a)',advance='no') 'Reading RDFs: '    
        call ReadRDF(FilRDF, NameMType, NBondMType, AdrSiteMType, NSType, NameSType,&! NA,&
            StateNB_iSjS, StateB_iBond, RDFref_B_riBond, rRDFB_riBond, RDFref_NB_riSjS, Bond_iSjS, & 
            NSiteGroupsInBond, iSiteInBond, jSiteInBond, kSiteInBond,&
            NPointsRDFNB, RMaxRDFNB, RMinRDFNB,&
            NPointsRDFB, RMaxRDFB, RMinRDFB)!, &
        write(stdout,*) 'RDFs are read successfully'        
        write(stdout,*) ''    
        write(stdout,*) '-----------------------------------------------------------------------'        
    endif       

    if (LReadPOT) then       
        write(stdout,*) ''    
        write(stdout,*) 'Reading potentials'    
        call ReadPot(FilPot, NameMType, NBondMType,AdrSiteMType,NSType, NameSType,&! NA,&
        StateNB_iSjS,StateB_iBond, PotB_riBond, rPotB_riBond,PotNB_riSjS, Bond_iSjS, & 
        NSiteGroupsInBond, iSiteInBond, jSiteInBond, kSiteInBond,&
        NPointsPotNB, RMaxPotNB, RMinPotNB,&
        NPointsPotB,RMaxPotB,RMinPotB)!, &
        write(stdout,*) 'Potentials are read successfully'        
        write(stdout,*) ''    
        write(stdout,*) '-----------------------------------------------------------------------'        
    endif
    
    if (LReadRDF.and.LReadPot) then
        if (NPointsPotNB.ne.NPointsRDFNB) then
            write(stdout,*) 'ERROR: Max number of points in NB RDF and potential differs:'
            write(stdout,*) 'NPointsPotNB=',NPointsPotNB, ' and  NPointsRDFNB=',NPointsRDFNB
            stop
        else
            NA=NPointsPotNB
        endif
    elseif (LReadRDF) then
        NA=NPointsRDFNB
    elseif (LReadPot) then
        NA=NPointsPotNB
    else
        write(stdout,*) 'ERROR: Niether RDF nor potential file were read'
        stop
    endif
    write(stdout,*) ''
    call SetParameters(NA, VOL, BOX, FRT, TEMP, FKJM, FPR, COULF, EPS, SHELV, RAS, NMol,NAtom,iAverage)
    write(stdout,*) ''
    write(stdout,*) '-----------------------------------------------------------------------'
    write(stdout,*) ''
    write(stdout,*) "Summarizing all RDF/potentials data which was read"

    call Initiate(NSType,NAtom, NactNB, NBond, NBondB, NBondA, BTypeBond, MTypeBond, NameMType, NA, POTCUT,FRT, &
        NameSType, &
        LReadRDF, LReadPot,&
        StateNB_iSjS,StateB_iBond,&
        RDFref_NB_riSjS,PotNB_RiSjS,RDFref_B_riBond, PotB_riBond,rPotB_riBond, iSiteInBond,jSiteInBond,&    
        IND, INDI, &
        NPointsRDFNB,NPointsPotNB, RMaxRDFNB, RMaxPotNB, RMinRDFNB, RMinPotNB,&
        NPointsRDFB,NPointsPotB,RMaxRDFB,RMaxPotB,RMinRDFB,RMinPotB, &
        LMoveMType, MTypeSType,Bond_iSjS,STypeAtom,MolAtom,SiteAtom,&
        IndexGlobNB_riSjS, IndexGlobB_riBond,AdrGlobIndexAtomij,&
        CORP, CORS, npairsrdf,LIMC, RCUT, RECUT,RDFINC)
    write(stdout,*) ''
    write(stdout,*) '-----------------------------------------------------------------------'
    write(stdout,*) ''
end subroutine ReadAll

subroutine SetParameters(NA, VOL, BOX, FRT, TEMP, FKJM, FPR, COULF, EPS, SHELV, RAS,&
                        NMol,NAtom,iAverage)!,&
    include "../dimpar.h"
    integer NA, NMol, NAtom, iAverage
    real(8) VOL, BOX(3), FRT, TEMP, FKJM, FPR, COULF, EPS
    real(8), allocatable:: SHELV(:), RAS(:)
    allocate(SHELV(NA),RAS(NA))
    VOL	= product(BOX)
    !VOL=BOXL*BOYL*BOZL
!    HBOXL=0.5*BOXL
	  
    FRT = 1.d-3*BOLTZ*TEMP*AVAG  !  En.: i.u -> kJ/M
    FKJM = FRT/NMol           
    FPR=1.d25*BOLTZ*TEMP/VOL                   !  Pr:  i.u -> atm
    !  electrostatic constant 
    COULF= 1.d10*ELCH**2/(4.d0*PI*EPS0*EPS*BOLTZ*TEMP)
    write(stdout,*)' COULF=',COULF,'  F(kt -> kJ) ',FRT,' P(N -> atm) ',FPR
    if (iAverage.le.0) iAverage=2*NAtom ! Set automatic value for averaging

#ifdef PBC_TO
    VOL=VOL*0.5
#endif
end subroutine SetParameters

subroutine GetFileLength(iFilename,iend, filetype)!, stdout)
    character(filename_text) iFilename
    character(len=*) filetype 
    character(input_text) l
    integer i, ierr, iend, iostt!, stdout
    ! Open the potential file and scan it  for number of lines (variable iend)
    open(unit=4,file=iFilename,status='old',iostat=iostt)
    if (iostt .ne. 0) then
        write(stdout,*)'ERROR: ',trim(filetype),' input file ',iFilename,' is not found!'
	stop
    endif
    i=0
    iend=0
    ierr=0
    do 
        read(4,'(a)',end=99) l
        iend=iend+1        
        cycle ! read next line
99      ierr=1 ! Return error flag when end of file
        if (ierr==1) then
            if (iPrint.ge.5) then
                write(stdout,*) 'End of the ',trim(filetype),' file reached.',iend,' lines read.'                
            endif
            exit
        endif
    enddo
    close(4)    
end subroutine

subroutine SetStdOut(stdout_, iPrint_)
        integer stdout_, iPrint_
        stdout=stdout_
        iPrint=iPrint_
        call mcmSetStdOut(stdout_, iPrint_)
        call rdfioSetStdOut(stdout_, iPrint_)
        call potioSetStdOut(stdout_, iPrint_)
end subroutine                

subroutine ReadCmdLine(rank, buffer)
    integer rank, iargc
    character(filename_text) buffer
    integer numargs
!  Reading command line arguments
    numargs=iargc()
    if (numargs.gt.1) then
        write(*,*) 'Too many parameters in command line:',numargs,'! Just one needed: Input file name.'
        stop
    elseif (numargs.eq.1) then
        call GETARG(1,buffer)
        if ((rank.eq.0)) write((6),*) 'Input file name stated in command line: ',buffer
    elseif(numargs.eq.0) then
        if ((rank.eq.0)) write((6),*) 'Input file name NOT stated in command line: Use default input.inp'
        buffer='input.inp'
    endif    
end subroutine ReadCmdLine

subroutine PrepareReadInput(rank, ID, psize, buffer, BaseOutFileName, text, iPrint)
    integer rank, ID, psize, iPrint
    character(filename_text) BaseOutFileName,OutFileName
    character(input_text), allocatable :: text(:)
    
    integer i, iend, iostt
    character(filename_text) buffer
    character(input_text) l
    
    call ReadCmdLine(rank ,buffer) ! read main input file name from the command line
    
    call SetStdOut(ID+6, 1)    
    if ((rank.eq.0)) then 
         write (*,*) ''
         write(*,*) '-----------------------------------------------------------------------'
         write(*,*) ' Reading main input file:',buffer
    endif
    call GetFileLength(buffer,iend,'main input')    ! Open the Input file and scan it  for number of lines (variable iend)
    call SetStdOut(ID+6, 5)
    ! Allocate array for the content of the file    
    allocate(text(iend))
 ! Read the file to the array text
    open(unit=4,file=buffer,status='old',iostat=iostt)
    do i=1, iend
        read(4,'(a)') l
        text(i)=trim(adjustl(l))
    enddo
    close(4)    
    call ReadParameter(iPrint, text, 'VerboseLevel', 'IPRINT',&
        'How much information to print',&
        'W',5)!,7,6)
    call ReadParameter(BaseOutFileName, text, 'Output', 'BASEOUTFILENAME',&
        'Base name for output files',&
        'W','output')!,iPrint,6)
    call SetStdOut(ID+6, iPrint)

  ! Open separate output file for each process
    if (ID.ne.0) then
        OutFileName=BaseOutFileName
        OutFileName(Len_Trim(OutFileName)+1:Len_Trim(OutFileName)+2)='.p'
        write(OutFileName(Len_Trim(OutFileName)+1:Len_Trim(OutFileName)+3),'(i0.3)') ID
        open(unit=ID+6,file=OutFileName,status='unknown')
        write(ID+6,*) 'Running in parallel mode: process ',ID,' of ',psize
    endif
end subroutine PrepareReadInput
end
