subroutine PotReadPropB(text, iFilename, bPotential, ePotential, NameMType, MolType,iBond, NBondMType,&
    AdrSiteMType, PRmin, PRmax, PNpoints,&
    Bond_iSjS, NSiteGroupsInBond, iSiteInBond, jSiteInBond, kSiteInBond, iprint, stdout)
    ! Global Variables
    character(pot_text), allocatable :: text(:)
    character(filename_text) iFilename
    character(mol_name_len) NameMType(:), NameMolType    
    integer, allocatable:: NBondMType(:),AdrSiteMType(:)
    integer bPotential, ePotential, PNPoints, bTable, eTable,  MolType, iBond, iprint, stdout
    integer, allocatable:: NSiteGroupsInBond(:), iSiteInBond(:,:), jSiteInBond(:,:), kSiteInBond(:,:),Bond_iSjS(:,:)
    real(8) PRmin, PRmax

! Local Variables
    integer  i,j,ierr, tmpNPairs, tmp, PNPairs, BondNumber,gBond
    integer, allocatable:: AtomPairs(:,:)
    character(pot_text) PairStr(2),field, l, str
    character(pot_text) PairsStr(pot_text)
    logical flag

    BondNumber=0
    NameMolType=''
    do j=bPotential, ePotential ! Read properties which are specific to Potential-type 
        call ReadLineFromText(text,j,str,ierr)
        l=str
        call split(l,'=',field)
        select case (uppercase(trim(adjustl(field))))
            case ('NPAIRS')
                call str2value(trim(adjustl(l)),PNPairs,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
                    ' line:',j,' Can not read the actual value of NPairs:',l
                    write(stdout,*) str
                    stop                
                endif
            case ('PAIRS')   
                call removesp(l)
                call parse(l,',',PairsStr,tmpNPairs)
                if (tmpNPairs.eq.0) then
                    write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),' line:',j,&
                    ' Can not read the list of atom pairs involved in the bond potential'
                    write(stdout,*) str
                    stop                
                endif
                if (tmpNPairs.ne.PNpairs) then
                    write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
                    ' line:',j,' Declared number of atomic pairs NPAIRS',PNpairs,&
                    ' differs from actual number of stated pairs ',tmpNPairs
                    write(stdout,*) str
                    stop                
                endif
                if (allocated(AtomPairs)) deallocate(AtomPairs)
                allocate(AtomPairs(tmpNPairs,3))
                do i=1, tmpNPairs
                    l=PairsStr(i)
                    call removesp(l)
                    call parse(l,'-',PairStr,tmp)
                    if (tmp.ne.2) then 
                        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
                        ' line:',j,' Can not read the atom numbers of the pair ',i,&
                        ' involved in the bond pair potential'
                        write(stdout,*) l
                        stop
                    endif
                    call str2value(trim(adjustl(PairStr(1))),AtomPairs(i,1),ierr)
                    call str2value(trim(adjustl(PairStr(2))),AtomPairs(i,2),ierr)
                    if (ierr.ne.0) then
                        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),' line:',j,&
                                   ' Can not read the atomic pair included in bond pair potential.',l
                        write(stdout,*) '"',trim(adjustl(PairStr(1))),'" ',' "',trim(adjustl(PairStr(2))),'"'           
                        stop
                    endif
                enddo
            case ('MOLTYPE')   
                call removesp(l)
                NameMolType=adjustl(trim(l))
                flag=.false.
                do i=1, size(NameMType)
                    if (adjustl(trim(NameMolType)).eq.adjustl(trim(NameMType(i)))) then 
                        flag=.true.
                        MolType=i
                    endif
                enddo
                if (.not. flag) then
                    write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
                        ' line:',j, ' unknown Molecular type ',NameMolType
                    stop
                endif
            case ('BONDNUMBER')
                call str2value(trim(adjustl(l)),BondNumber,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),' line:',j, &
                            ' Can not read the actual number of the bond ',l
                    write(stdout,*) str
                    stop                
                endif
            case default
                continue !write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
        end select
    enddo
    if (BondNumber.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
        ' Lines:',bPotential,':', ePotential, ' BondNumber is not stated' 
        stop
    endif
    if (adjustl(trim(NameMolType)).eq.'') then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
        ' Lines:',bPotential,':', ePotential, ' Molecular type name is not stated' 
        stop
    endif
    gBond=sum(NBondMType(1:MolType-1))
    iBond=sum(NBondMType(1:MolType-1))+BondNumber    
    if (PNPairs.ne.NSiteGroupsInBond(iBond)) then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
            ' Potential for bond ', BondNumber, ' of MolType ', trim(NameMType(MolType))
        write(stdout,*) 'Number of site/atom groups defined for the bond: ',PNPairs, ' differs from'
        write(stdout,*) 'number of site/atom groups defined in the mcm file : ', NSiteGroupsInBond(iBond)
        stop
    endif
        
    do i=1, PNPairs ! Pairs stated in the Potential-file        
        flag=.false.
        if (Bond_iSjS(AtomPairs(i,1)+AdrSiteMType(MolType),AtomPairs(i,2)+AdrSiteMType(MolType)).ne.iBond) then
            write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),' Bond-length distribution for bond '&
                    , BondNumber, 'of MolType ', trim(NameMType(MolType))
            write(stdout,*) 'According to the molecular description in mcm-file, site pair:',AtomPairs(i,1),AtomPairs(i,2),&
            'does not belong to the bond ',BondNumber
!            write(stdout,*) Bond_iSjS(AtomPairs(i,1)+AdrSiteMType(MolType),AtomPairs(i,2)+AdrSiteMType(MolType)), iBond
            stop            
        endif
        do j=1, NSiteGroupsInBond(iBond) ! Pairs stated in mcm-file
            if (AtomPairs(i,1)+AdrSiteMType(MolType).eq.iSiteInBond(iBond,j).and.&
                AtomPairs(i,2)+AdrSiteMType(MolType).eq.jSiteInBond(iBond,j))&
!                AtomPairs(i,3).eq.kSiteInBond(iBond,j)) &
                then
                flag=.true.
                exit
            endif
        enddo
        if (.not.flag) then
            write(stdout,*) 'ERROR: Reading Potential: File: ',trim(iFilename),&
                ' Bond distribution for bond ', BondNumber, 'of MolType ', trim(NameMType(MolType))
            write(stdout,*) 'Can not find ',i,' group of sites/atoms defining the bond in the mcm file : ', AtomPairs(i,1:2)
                    stop
        endif
    enddo
end subroutine

!    character(pot_text), allocatable :: text(:)
!    character(filename_text) iFilename
!    character(pot_text) PName
!    integer bPotential, ePotential, PNPoints, bTable, eTable, PNPairs
!    integer, allocatable:: AtomPairs(:,:)
!    real PRmin, PRmax
!
!! Local Variables
!    integer  i,j,ierr, tmpNPairs, tmp
!    character(pot_text) PairStr(2),field, l, str
!    character(pot_text) PairsStr(pot_text)
!    
!    do j=bPotential, ePotential ! Read properties which are specific to potential-type 
!        call ReadLineFromText(text,j,str,ierr)
!        l=str
!        call split(l,'=',field)
!        select case (uppercase(trim(adjustl(field))))
!            case ('NPAIRS')
!                call str2value(trim(adjustl(l)),PNPairs,ierr)
!                if (ierr.ne.0) then
!                    write(stdout,*) 'ERROR: Reading Potential: line:',j,' Can not read the actual value of NPairs:',l
!                    write(stdout,*) str
!                    stop                
!                endif
!            case ('PAIRS')   
!                call parse(l,',',PairsStr,tmpNPairs)
!                if (tmpNPairs.eq.0) then
!                    write(stdout,*) 'ERROR: Reading Potential: line:',j,' Can not read the list of atom pairs involved in bond'
!                    write(stdout,*) str
!                    stop                
!                endif
!                if (tmpNPairs.ne.PNpairs) then
!                    write(stdout,*) 'ERROR: Reading Potential: line:',j,' Declared number of atomic pairs NPAIRS',PNpairs,&
!                    ' differs from actual number of stated pairs ',tmpNPairs
!                    write(stdout,*) str
!                    stop                
!                endif
!                if (allocated(AtomPairs)) deallocate(AtomPairs)
!                allocate(AtomPairs(tmpNPairs,2))
!                do i=1, tmpNPairs
!                    l=PairsStr(i)
!                    call removesp(l)
!                    call parse(l,'-',PairStr,tmp)
!                    if (tmp.ne.2) then 
!                        write(stdout,*) 'ERROR: Reading Potential: line:',j,' Can not read the atom numbers of the pair ',i,&
!                        ' involved in the bond'
!                        write(stdout,*) l
!                        stop
!                    endif
!                    call str2value(trim(adjustl(PairStr(1))),AtomPairs(i,1),ierr)
!                    call str2value(trim(adjustl(PairStr(2))),AtomPairs(i,2),ierr)
!                    if (ierr.ne.0) then
!                        write(stdout,*) 'ERROR: Reading Potential: line:',j,&
!                                   ' Can not read the atomic pair interacting by the potential.',l
!                        write(stdout,*) '"',trim(adjustl(PairStr(1))),'" ',' "',trim(adjustl(PairStr(2))),'"'           
!                        stop
!                    endif
!                enddo                
!            case default
!                write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
!        end select
!    enddo    
!end subroutine
