subroutine ReadIncludedPotentialSection(IncludeFileName, text, iend, bInclude, eInclude, bTable, eTable)

!! Global variables
     character(filename_text) IncludeFileName
     integer iend, bInclude, eInclude
     integer bTable, eTable
     character(pot_text), allocatable :: text(:)
!! Local variables   
     integer i,j, cnt_bInclude, cnt_eInclude, ierr, iostt, cnt_eTable, cnt_bTable
     character(pot_text) l,str,field
     logical FlagTableFound
    
    ! Open the Potential file and scan it  for number of lines (variable iend)    
    call GetPotFileLength(IncludeFileName,iend)
! Allocate array for the file    
    if (allocated(text)) deallocate(text)
    allocate(text(iend))
! Read the file to the array text
    open(unit=44,file=IncludeFileName,status='old',iostat=iostt)
    do i=1,iend
        read(44,'(a)') l
        text(i)=trim(adjustl(l))
!        write(stdout,*) text(i)
    enddo
    close(44)
    i=1
    bInclude=0
    eInclude=0
    cnt_bInclude=0
    cnt_eInclude=0
    cnt_eTable=0
    cnt_bTable=0
    do while (i.le.iend)
        call ReadLineFromText(text,i,l,ierr)
        ! Indicate presence of &Include 
        if(uppercase(trim(l))=='&INCLUDEDPOTENTIAL') then
            bInclude=i
            cnt_bInclude=cnt_bInclude+1
        endif
        if(uppercase(trim(l))=='&ENDINCLUDEDPOTENTIAL') then 
            eInclude=i
            cnt_eInclude=cnt_eInclude+1
        endif
        i=i+1
    enddo
    ! Indicate presence of &EndInclude and check consistency
    if ((cnt_bInclude.gt.1) .or. (cnt_eInclude.gt.1)) then
        write(stdout,*) 'ERROR: Reading Potential: file ',trim(IncludeFileName), &
        ' more than one line with tag &IncludedPotential or &EndIncludedPotential found '
        stop
    endif
    if (bInclude.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: file ',trim(IncludeFileName),&
            ' Can not find begin of section &IncludedPotential'
        stop
    endif
    if (eInclude.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: file ',trim(IncludeFileName),&
            'Can not find end of section &EndIncludedPotential'
        stop 
    endif
    if (bInclude.gt.eInclude) then
        write(stdout,*) 'ERROR: Reading Potential: file ',trim(IncludeFileName), &
        ' The section &IncludedPotential starts at line ',bInclude, 'and ends at line ',eInclude
        stop
    endif
    
!        write(stdout,*) 'Reading Included Potential file'
    do j=bInclude, eInclude
        call ReadLineFromText(text,j,str,ierr)
        l=str
        call split(l,'=',field)        
        select case (uppercase(trim(adjustl(field))))
             case ('&TABLE')
                   FlagTableFound=.true.
                   bTable=j
                   cnt_bTable=cnt_bTable+1
            case ('&ENDTABLE')
                   eTable=j
                   cnt_eTable=cnt_eTable+1                        
            case default
                !write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
                continue
        end select                                
    enddo
    ! Check consistency of &Table &EndTable tags
    if  ((cnt_bTable.gt.1) .or. (cnt_eTable.gt.1)) then
        write(stdout,*) 'ERROR: Reading Included Potential : ',trim(IncludeFileName),&
                ': more than one line with tag &Table or &EndTable found '
        stop
    endif
    if (bTable.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(IncludeFileName),': Can not find begin of section &Table'
        stop
    endif
    if (eTable.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(IncludeFileName),' Can not find end of section &EndTable'
        stop 
    endif
    if (bTable.gt.eTable) then
        write(stdout,*) 'ERROR: Reading Potential: File: ',trim(IncludeFileName),&
                    ': The section &Table starts at line ',bTable, 'and ends at line ',eTable
        stop
    endif         
end subroutine

subroutine ReadIncludePotentialSection(text,iend,IncludeFileName,Npoints,RMax,RMin,Name,Type, bTable,eTable)
! Global variables
    integer bTable, eTable
    integer NPoints, iend
    character(pot_text), allocatable :: text(:)
    character(pot_text) Type, IncludeFileName, name
    real  RMin, RMax
! Local variables        
    integer i,j,k, ierr, iostt, cnt_bInclude, cnt_eInclude,bInclude, eInclude,PNPoints,&
            cnt_eTable, cnt_bTable
    character(pot_text) l,str,field, PType, Pname
    real PRMax, PRMin
    logical FlagTableFound
    i=1
    bInclude=0
    eInclude=0
    cnt_bInclude=0
    cnt_eInclude=0
    do while (i.le.iend)
        call ReadLineFromText(text,i,l,ierr)
        ! Indicate presence of &Include 
        if(uppercase(trim(l))=='&INCLUDEDPOTENTIAL') then
            bInclude=i
            cnt_bInclude=cnt_bInclude+1
        endif
        if(uppercase(trim(l))=='&ENDINCLUDEDPOTENTIAL') then 
            eInclude=i
            cnt_eInclude=cnt_eInclude+1
        endif
        i=i+1
    enddo
    ! Indicate presence of &EndInclude and check consistency
    if ((cnt_bInclude.gt.1) .or. (cnt_eInclude.gt.1)) then
        write(stdout,*) 'ERROR: Reading Potential: more than one line with tag &IncludedPotential',&
            ' or &EndIncludedPotential found '
        stop
    endif
    if (bInclude.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: Can not find begin of section &IncludedPotential'
        stop
    endif
    if (eInclude.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: Can not find end of section &EndIncludedPotential'
        stop 
    endif
    if (bInclude.gt.eInclude) then
        write(stdout,*) 'ERROR: Reading Potential: The section &IncludedPotential starts at line ',&
            bInclude, 'and ends at line ',eInclude
        stop
    endif
            
    ! Read fields
    ! Default values    
    PRMax=-1
    PRMin=-1
    PNPoints=0
    write(stdout,*) 'Reading Included potential file'
    do j=bInclude, eInclude
        call ReadLineFromText(text,j,str,ierr)
        l=str
        call split(l,'=',field)        
        select case (uppercase(trim(adjustl(field))))
            case ('NAME')
                PName=trim(adjustl(l))
!                       write(stdout,*) 'Debug: Name:',Pname
            case ('MIN')
                call str2value(trim(adjustl(l)),PRMIN,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,& 
                                ' Can not read the actual value of MIN:',l
                    write(stdout,*) str
                    stop                                                                                                    
                endif
                if (PRMIN.ne.RMIN) then
                     write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' The stated value of MIN:',PRMIN,' differs from the one in the general potential-file: ',RMIN                    
                    stop                                                                      
                endif
!                      write(stdout,*) 'Debug: RMIN:',Prmin
            case ('MAX')
                call str2value(trim(adjustl(l)),PRMAX,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' Can not read the actual value of MAX:',l
                    write(stdout,*) str
                    stop                                                                      
                endif
                if (PRMAX.ne.RMAX) then
                     write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' The stated value of MAX:',PRMAX,' differs from the one in the general potential-file: ',RMAX                    
                    stop                                                                      
                endif
!                       write(stdout,*) 'Debug: RMAX:',Prmax
            case ('NPOINTS')
                call str2value(trim(adjustl(l)),PNPoints,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' Can not read the actual value of NPoints:',l
                    write(stdout,*) str
                    stop                
                endif                
                if (PNPOINTS.ne.NPOINTS) then
                     write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' The stated value of NPoints:',PNPOINTS,' differs from the one in the general potential-file: ',NPOINTS                    
                    stop                                                                      
                endif
            case ('TYPE')
                PType=trim(adjustl(l))
!                write(stdout,*) 'Debug Type=',PType
                select case (trim(adjustl(PType)))
                    case ('NB')
                        continue
                    case ('B')
                        continue
                    case ('A')
                        continue
                    case default
                        write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',&
                                    j,': Wrong type of the potential:',trim(adjustl(PType)),'    It shall be NB, B or A'                                
                end select 
                if ((trim(adjustl(PType))).ne.(trim(adjustl(Type)))) then
                     write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': line:',j,&
                            ' The stated value of PTYPE:',(trim(adjustl(PType))),&
                            ' differs from the one in the general potential-file: ',(trim(adjustl(Type)))                    
                    stop                                                                      
                endif
!                        write(stdout,*) 'Debug: NPOINTS:',PNPOINTS
             case ('&TABLE')
                   FlagTableFound=.true.
                   bTable=j
                   cnt_bTable=cnt_bTable+1
            case ('&ENDTABLE')
                   eTable=j
                   cnt_eTable=cnt_eTable+1                        
            case default
                continue !write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
        end select                                
    enddo
    ! Check consistency of &Table &EndTable tags
    if  ((cnt_bTable.gt.1) .or. (cnt_eTable.gt.1)) then
        write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),&
                ': more than one line with tag &Table or &EndTable found '
        stop
    endif
    if (bTable.eq.0) then
        write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),': Can not find begin of section &Table'
        stop
    endif
    if (eTable.eq.0) then
        write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),' Can not find end of section &EndTable'
        stop 
    endif
    if (bTable.gt.eTable) then
        write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName),&
                    ': The section &Table starts at line ',bTable, 'and ends at line ',eTable
        stop
    endif   
   ! Check if all the mandatory parameters are set
   if (NPoints.eq.0) then
       write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName)
       write(stdout,*) 'Number of points (NPOINTS) is not defined.'       
       stop
   endif
   if (PRMAX.le.-0.0000001) then
       write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName)
       write(stdout,*) 'MAX is not defined'       
       stop
   endif
   if (PRMIN.le.-0.00000001) then
       write(stdout,*) 'ERROR: Reading Included Potential: ',trim(IncludeFileName)
       write(stdout,*) 'MIN is not defined'       
       stop
   endif
   
end subroutine
