! CollectMCsummary - subroutine collects statistic data from all processes with MPI, and the calculates final averaged values.
! PrintMCsummary - subroutine prints final output in Molsim (priorly collected )

module magicmod
    use utilsmod
    implicit none
    contains
    include 'out.f'
    subroutine InitMagic(psize, rank, ID)    
        use timer
        implicit none
        integer(4) psize, rank, ID
#ifdef mpi
    include 'mpif.h'
#endif    
        integer ierror
        character*64 LABEL,SControl*64,DATE*64! Labels for program version and date (not used)

        call TimerInit() ! Start timer        
    !Init MPI, detecting number of processes and the process ID
#ifdef mpi
        call TimerStart(T_MPI)
        call MPI_INIT(ierror)
        call MPI_COMM_SIZE(MPI_COMM_WORLD,psize,ierror)
        call MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierror)
        call TimerStop(T_MPI)
#endif
        if (psize.ge.1) ID=ID+1+rank ! Set ID if in parallel mode
        if ((psize.ge.1).and.(rank.eq.0)) then
            write((6),*) 'Running in parallel mode on ',psize,' processes'
        elseif ( psize .eq. 0) then
            write((6),*) 'Running in serial mode'
        endif

     ! Version identification
	include 'version.h'
!        LABEL='MagiC_v.2.2'
!        DATE='09.09.15'
        if ((rank.eq.0)) then
            write((6),*)'======================================'
            write((6),*)' Molecular Simulation Program MagiC '
            write((6),*)'  General output '
            write((6),'(4a)')'  Version ',trim(LABEL)
            write((6),'(2a)')'  from ',trim(DATE)
	    write((6),'(a)') ' source control:', trim(SCONTROL)
            write((6),*)'======================================'
            write((6),*) 'Thank',"'",'s for citing the software paper:'
            write((6),*)'A. Mirzoev, A.P.Lyubartsev, MagiC: Software Package for Multiscale Modeling'
            write((6),*)'J. Chem. Theory and Computations, 9(3), 1512-1520 (2013). DOI:10.1021/ct301019v'
            write((6),*)'======================================'
#ifdef PBC_MULT
            write((6),*)' PBC_Mult enabled ... '
#endif        
            write((6),*)' starting ... '
        endif    

    end subroutine

    subroutine ResetCounters(stdout, NAV, AccCntMType, ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES, CORS, CORP,npairsrdf)
        implicit none
        integer(4) NAV, stdout
        integer(8), allocatable:: npairsrdf(:,:), CORS(:), CORP(:,:),AccCntMType(:,:,:)
        real(8) ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES
        !  New run, summators=0
            write((stdout),*) 'Set all counters to zero'
            NAV=0            
            AccCntMType(:,1,1:3)=0
            AccCntMType(:,2,1:3)=1
            ENERS=0.d0
            EELS=0.
            EINTS=0.
            VIRS=0.
            VIIS=0.
            VIRE=0.
            PRES=0.
            CORS(:)=0.d0            
            if (allocated(CORP)) CORP(:,:)=0.d0
            npairsrdf(:,:)=0

    end subroutine

    !===================== READCRD ==================================
    ! read initial coordinates and eventually velocities from a "xmol" file s
    subroutine READCRD(Svec,NAtom,SiteAtom,FSTART,NFStart,ID, stdout)        
        use precision
        implicit none
    ! Global Variables    
        real(8), intent(out) ::Svec(:,:)
        integer(4), intent(in):: NAtom,ID, stdout, NFStart
        integer(4), intent(in)::SiteAtom(:)
        character(filename_text), intent(in):: FSTART    
    ! Local Variables
        character(filename_text) LoadFileName ! Local variable to work with, without changing global FSTART
        character*80 CHR
        integer(4) i,iostat,IS,NPP, NConf

        LoadFileName=FSTART
        write((stdout),*) ' Read starting geometry from file ',LoadFileName
        if ((NFStart.eq.0).and.(ID.ne.0)) then ! If each process read only one configuration from individual file
            LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+2)='.p'
            write(LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+3),'(i0.3)') ID
        endif
        LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+11)='.start.xmol'
        open(unit=4,file=LoadFileName,status='old',iostat=iostat)
        if (iostat.ne.0) then
            if (ID.gt.0) write(stdout,*) ' Problems reading file ',trim(LoadFileName),' on process ID:',ID
            write(6,*) ' Problems reading file ',trim(LoadFileName)
            stop
        else
            write((stdout),*) ' The coordinates file was opened for reading: ',LoadFileName
        endif
        
        ! Generate random number Nconf and skip first NConf*NAtom
        if (NFStart.gt.0) then
            NConf=RAND()*(NFStart)*0.999999
        else
            NConf=0
        endif
        write(stdout,*) 'Will use configuration ',NConf+1,' from file ',trim(LoadFileName)
        ! Skipping lines
        do i=1, (NAtom+2)*NConf
            read(4,*) CHR
        enddo
        write(stdout,*) 'Reading ',NConf+1,' configuration from file ',trim(LoadFileName)
        ! Now we are reading it for real
        read(4,*)NPP
        if(NPP.ne.NAtom)then        
            write(stdout,*) ' wrong N mol. in CRD file ',LoadFileName,' Sholud be:',NAtom,' In file:',NPP,' ID:',ID        
            stop
        end if
        read(4,*)CHR
        do I=1,NAtom
            read(4,*)CHR,Svec(:,i)
        end do
        write(stdout,*)' Initial coord.  read from ',LoadFileName
        close(4)
    end subroutine READCRD

    subroutine InitGeometry(LCRD, FSTART,NFStart, FCRD, LCRDPASS, LMoveMType, ID, stdout, iPrint, iREP, NAtom,&
        NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,NameMType,&
        XvecCOMMol,Svec, Wvec, RSite,BOX,Ri_vec)

        use precision
        implicit none
        ! Global
        logical LCRD,  LCRDPASS, LMoveMType(:)
        integer(4)  ID, stdout, iPrint, iREP, NAtom, NSite
        integer(4), intent(in):: NFSTart
        character(filename_text) FSTART, FCRD
        character(mol_name_len) NameMType(:)
        integer(4) NMType, NMolMType(:), NSiteMType(:),SiteAtom(:), AdrAtomMType(:), AdrMolMType(:),AdrAtomMol(:)
        real(8) XvecCOMMol(:,:), Svec(:,:), Wvec(:,:), RSite(:,:),BOX(3), Ri_vec(:,:)
        !Local
        integer(8) i
        !  Preparing initial state geometry

        write(stdout,*) '' 
        write(stdout,*) 'Prepare starting geometry' 

        if (IREP.eq.1) then  ! The first iteration
            write(stdout,*) 'The first inverse iteration'
            if (LCRD) then
                write(stdout,*) 'Reading initial geometry from file' 
                call READCRD(Svec,NAtom,SiteAtom,FSTART,NFStart,ID,stdout)
            else            
                write(stdout,*) 'Randomly generate initial geometry' 
                call GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,&
                    LMoveMType, NameMType,&
                        XvecCOMMol, Svec, Wvec,RSite,BOX, FCRD, iPrint, stdout )
            endif
            Ri_vec=Svec
        else ! Not the first iteration
            if (.not. LCRDPASS) then
                if (LCRD) then
        	    write(stdout,*) 'Reading initial geometry from file' 
        	    call READCRD(Svec,NAtom,SiteAtom,FSTART,NFStart,ID,stdout)
        	else            
            	    write(stdout,*) 'Randomly generate initial geometry' 
            	    call GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,&
                	    LMoveMType, NameMType,&
                    	    XvecCOMMol,Svec, Wvec, RSite,BOX, FCRD, iPrint, stdout )
        	endif                
                Ri_vec=Svec
                else
                write(stdout,*) 'Continue with the existing geometry of the system'             
            endif
        endif        
    end subroutine

    subroutine GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType, AdrAtomMol,LMoveMType,&
        NameMType, XvecCOMMol, Svec, Wvec, RSite, BOX, FCRD, iPrint, stdout)
        use utilsmod
        use precision
        implicit none
        ! Global
        integer(4) NMType, NMolMType(:), NSiteMType(:),SiteAtom(:), AdrAtomMType(:), AdrMolMType(:),AdrAtomMol(:), iPrint, stdout
        logical LMoveMType(:)
        character(filename_text) FCRD    
        character(mol_name_len) NameMType(:)
        real(8) XvecCOMMol(:,:),Svec(:,:),Wvec(:,:), RSite(:,:),BOX(3)
        !Local
        integer(4) iMType, iostt, IC, iMol, iAtom, im ,is, isp
        real(8) QP(4)        ! quaternion array
        character(atom_name_len) atomname

    ! If there are fixed molecules:
        if (.not. all(LMoveMType)) then
            write(stdout,*) 'There are frozen molecules in the system, their positions will be read from file: ', trim(FCRD)
            open (unit=3,file=FCRD,status='old',iostat=iostt)
            if (iostt.ne.0) stop 'The coordinate file not found'
            read(3,*)
            read(3,*)
        endif

        do iMType=1,NMType
            if(.not.LMoveMType(iMType)) then  !  fixed coordinates (e.g.DNA)        
                IC=AdrAtomMType(iMType)
                do iMol=1,NMolMType(iMType)
                    do iAtom=1,NSiteMType(iMol)
                        IC=IC+1                            
                        read(3,*) atomname,Svec(:,ic)
                    end do
                end do
            else
                write(stdout,*)' Generate random distribution of molecules ',trim(NameMType(iMType))
    !  random distribution for COM particles
                do iMol=1,NMolMType(iMType)
                    IM=AdrMolMType(iMType)+iMol                    
                    XvecCOMMol(:,im)=([rand(), rand(), rand()]-0.5d0)*BOX
                    if(IPRINT.ge.7)write(stdout,'(I6,3F12.6)')IM,XvecCOMMol(:,IM)!,Y(IM),Z(IM)
    ! random rotation - using quaternion
                    call RANQ(QP)
                    if(IPRINT.ge.7)write(stdout,'(4F12.6)') QP(1),QP(2),QP(3),QP(4)                                
                    do iatom=1,NSiteMType(iMType)
                        ISP=AdrAtomMol(IM)+iAtom
                        IS=SiteAtom(ISP)
                        Wvec(1:3,isp)=Rsite(IS,1:3)
                        call ROTATE(QP,Wvec(1,ISP),Wvec(2,ISP),Wvec(3,ISP)) ! This one rotates molecule on quaternion QP                        
                        Svec(:,isp)=XvecCOMMol(:,IM)+Wvec(:,isp)
                        if(IPRINT.ge.7)write(stdout,'(3I6,3F12.4)') IM,IS,ISP,Svec(:,isp)
                    end do
                end do
            end if  ! LMoveMType
        end do  ! NMType
        if (.not. all(LMoveMType)) close(3)
    end subroutine
    
    subroutine TerminateMagiC(PotCorrJourn,SumMassMType,PotNB_riSjS,PotB_riBond,&
            XvecCOMMol,RSite,Svec, Wvec, MassSite, RMinPotB,RMaxPotB,&
            NSiteMType,AdrBondMType,AdrSiteMType,AdrMolMType,AdrAtomMType,AdrAtomMol,&
            NBondMType,Bond_iSjS,NPairsTotalBond,NSiteGroupsInBond, IndexGlobNB_riSjS,IndexGlobB_riBond,IND,INDI,&
            MTypeSType, MTypeMol, MTypeSite,&
            iSiteinBond,jSiteinBond, kSiteinBond,NPointsPotB,NMIN, MolAtom,STypeAtom, SiteAtom, AtomChrgAtom,&
            CORS,CORP,npairsrdf,global_npairsrdf,global_CORP, global_CORS,& 
            NameSType, NameSite,ID, stdout,psize)
        use precision
        use timer
        implicit none
#ifdef mpi
        include 'mpif.h'
#endif        
        
        real(8), allocatable:: PotCorrJourn(:,:),SumMassMType(:),PotNB_riSjS(:,:,:),PotB_riBond(:,:),&
            XvecCOMMol(:,:),RSite(:,:),Svec(:,:),Wvec(:,:),&
            MassSite(:), RMinPotB(:),RMaxPotB(:)
        integer(4), allocatable :: NSiteMType(:),AdrBondMType(:),AdrSiteMType(:),AdrMolMType(:),AdrAtomMType(:),AdrAtomMol(:),&
            NBondMType(:),Bond_iSjS(:,:),NPairsTotalBond(:),NSiteGroupsInBond(:), &
            IndexGlobNB_riSjS(:,:,:),IndexGlobB_riBond(:,:),IND(:,:,:),INDI(:,:),MTypeSType(:), MTypeMol(:), MTypeSite(:),&
            iSiteinBond(:,:),jSiteinBond(:,:), kSiteinBond(:,:),NPointsPotB(:),NMIN(:), MolAtom(:),&
            STypeAtom(:), SiteAtom(:), AtomChrgAtom(:)
        integer(8), allocatable:: CORS(:),CORP(:,:),npairsrdf(:,:),global_npairsrdf(:,:),global_CORP(:,:), global_CORS(:) 
        
        character(atom_name_len), allocatable:: NameSType(:), NameSite(:)
        integer(4) ierror, stdout, ID, psize
        
        if (allocated(PotCorrJourn)) deallocate(PotCorrJourn)
        deallocate(NSiteMType,AdrSiteMType,AdrMolMType,AdrAtomMType,NBondMType,AdrBondMType,SumMassMType)
        deallocate(PotNB_riSjS,IndexGlobNB_riSjS,IND,NameSType,MTypeSType)
        deallocate(AdrAtomMol,MTypeMol,XvecCOMMol) ! Depends on total number of molecules (former NPART)
        deallocate(RSite,NameSite,MassSite,Bond_iSjS,MTypeSite,NPairsTotalBond) !Depends on total number of sites (former NSAM)
        if (allocated(NSiteGroupsInBond)) then 
            deallocate(NSiteGroupsInBond)
            deallocate(iSiteinBond,jSiteinBond, kSiteinBond)
        endif
        deallocate(RMinPotB,RMaxPotB,NPointsPotB,NMIN) ! Depends on number of intramolecular potentials (former NSPAR)
        if (allocated(INDI)) deallocate(INDI)
        deallocate(IndexGlobB_riBond,PotB_riBond)
        deallocate(CORS)
        if (allocated(CORP)) deallocate(CORP)
        if (ID.eq.1) then  ! If in parallel mode on master process
            deallocate(global_CORS)
            deallocate(global_npairsrdf)
            if (allocated(global_corp)) deallocate(global_CORP)
        endif
        deallocate(MolAtom,STypeAtom,SiteAtom,AtomChrgAtom) ! Depends on total number of atoms (former NTOT)
        deallocate(Svec,Wvec)!!Depends on total number of atoms (former NTOT)
        deallocate(npairsrdf)
        call TimerReport(stdout) ! local report on every parallel process
        if (ID.ne.0) then
            close(stdout)
        endif
#ifdef mpi
!        call TimerStart(T_MPI)
        call MPI_BARRIER(MPI_COMM_WORLD,ierror)        
        call TimerCollectMPI(psize)
        call MPI_FINALIZE(ierror)
!        call TimerStop(T_MPI)
#endif

        if (ID.le.1) then 
            call TimerStamp(6,'MagiC ends:')
            write(6,*) ''
            write(6,*) 'Average timing (over all processes):'
            call TimerReport(6)
        endif
    end subroutine TerminateMagiC
    
!
!=================== Collect =====================================
!   Subroutine collecting data from different processes using MPI
subroutine Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP,&
	npairsrdf,global_npairsrdf,LIMC, NMType)
use dynarray

!-------Global Variables--------------------
integer(4) ID, ierror,rank,psize,NAV,global_NAV, NMType
integer(8), allocatable:: CORS(:),CORP(:,:),global_CORS(:),global_CORP(:,:),&
	npairsrdf(:,:),global_npairsrdf(:,:),global_AccCntMType(:,:,:)
logical LIMC
!-------Local Variables--------------------
integer(4) i,j,k
#ifdef mpi
include 'mpif.h'

!    if (ID.eq.1) then  ! If in parallel mode on master process
!        call reshape(global_CORS,size(CORS))
!        if (LIMC) call reshape2(global_CORP,size(CORP,1),size(CORP,2))
!        call reshape2(global_npairsrdf,size(npairsrdf,1),size(npairsrdf,2))
!    endif

!#ifdef intel
    if (ID.ge.1) then  ! If in parallel mode, allocate memory on all processes. for intel compilers
        call reshape(global_CORS,size(CORS))        
        if (LIMC) call reshape2(global_CORP,size(CORP,1),size(CORP,2))
        call reshape2(global_npairsrdf,size(npairsrdf,1),size(npairsrdf,2))
    endif
!#endif        

    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Gathering CORS - self-correlated terms
!   write(stdout,*) 'AllGather CORS started'
    call MPI_REDUCE(CORS,global_CORS,size(CORS),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
 !   write(stdout,*) 'AllGather CORS done'
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Gathering CORP - cross-correlated terms
    if (LIMC) then ! If IMC mode, collect cross correlated terms
        call MPI_REDUCE(CORP,global_CORP,size(CORP),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    endif
! Gathering number of counted pairs in RDF calc
    call MPI_REDUCE(npairsrdf,global_npairsrdf,size(npairsrdf),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(NAV,global_NAV,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    if (ID.eq.1) then
        if (global_NAV.ne.(NAV*psize)) write(*,*) 'MPI ERROR in NAV - reduction', global_NAV,'!=',NAV*psize
    endif
#endif
end subroutine

#ifdef mpi
Subroutine CollectMCsummary(NAV,global_NAV,MKSteps,global_MKSteps,AccCntMType,global_AccCntMType,EELS,global_EELS,EINTS,&
                global_EINTS, ENERS,global_ENERS,VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE,LRDF,ID,psize,ierror)
    use dynarray
    include 'mpif.h'
!Global variables
    integer(4) NAV,global_NAV,&                
                psize,ierror,ID
    integer(8) AccCntMType(:,:,:),MKSteps,global_MKSteps
    integer(8), allocatable:: global_AccCntMType(:,:,:)
    real(8) EELS,global_EELS,&
            EINTS,global_EINTS,&
            ENERS,global_ENERS,&
            VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE
    logical LRDF
! Local variables
    integer(4) i
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
    if (.not.allocated(global_AccCntMType)) &
        allocate(global_AccCntMType(size(AccCntMType,1),size(AccCntMtype,2),size(AccCntMtype,3)))
    global_AccCntMType=0
! Collect NAV
    call MPI_REDUCE(NAV,global_NAV,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect MKSteps
    call MPI_REDUCE(MKSteps,global_MKSteps,1,MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect AccCntMType(:)
    call MPI_REDUCE(AccCntMType,global_AccCntMType,size(AccCntMType),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)    
! Collect EELS
    call MPI_REDUCE(EELS,global_EELS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect EINTS
    call MPI_REDUCE(EINTS,global_EINTS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect ENERS
    call MPI_REDUCE(ENERS,global_ENERS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect PRESSURE PRES=NAtom*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*FNR
    call MPI_REDUCE(VIRS,global_VIRS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIIS,global_VIIS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIRE,global_VIRE,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)

    if (ID.eq.1) then ! Calculate mean values and write debug/perform check
        if (global_NAV.ne.(NAV*psize)) write(*,*) 'MPI ERROR in NAV - reduction', global_NAV,'!=',NAV*psize
        if (global_MKSteps.ne.(psize*MKSteps))&
            write(*,*)'MPI ERROR in MKSteps-reduction, global_MKSteps!=MKSteps*psize', global_MKSteps,'!=',MKSteps*psize
    endif
end subroutine
#endif


! This subroutine prints final output in MagiC (priorly collected )
subroutine PrintMCsummary(stdout,NAV,iOUTFactor,NMType,NMol,MKSteps,NMCStepsEquil,NAtom,ITRANS,IROT,NMolMType,AccCntMType,&
        BOX,MCStepAtomMType,MCTransStepMType,MCRotStepMType,TEMP,EPS,EELS,EINTS,&
        ENERS,VIRS,VIRE,VIIS,FPR,LRDF,NameMType,FILRDF,FILPOT)
!Global variables
    use mcmod
    implicit none
    integer(4), intent(in):: stdout,NAV,NMType,NMol,NAtom,ITRANS,IROT
    integer(4), intent(in):: NMolMType(:)
    integer(8), intent(in):: AccCntMType(:,:,:),MKSteps,NMCStepsEquil
    real(8), intent(in):: BOX(3),TEMP,EPS,EELS,EINTS,ENERS,VIRS,VIRE,VIIS,FPR, iOUTFactor
    real(8), intent(in) :: MCStepAtomMType(:),MCTransStepMType(:),MCRotStepMType(:)
            
    logical LRDF    
    character(filename_text) FILRDF,FILPOT
    character(mol_name_len), intent(in):: NameMType(:)
! Local variables
    real(8) FNR, PRES
    integer(4) I 
    character(len=5) :: fmt !Auxilary variable for output format specification
    
    write((stdout),*)'MagiC '
        write((stdout),*)' Monte Carlo'

    write((stdout),*)'-----------------------------------'
    if(LRDF)then
        write((stdout),*)' Inverse simulation'
        write((stdout),*)' File with original RDF:',FILRDF
    else
        write((stdout),*)' Direct simulation'
        write((stdout),*)' Potential is read from ',FILPOT
    end if
    if(NAV.gt.0)then
        FNR=1./dfloat(NAV)
    else
        FNR=0.
    end if
    write((stdout),*)'Number of molecules:   ',NMol
    write((stdout),*)'Number of species:     ',NMType
    write((stdout),'(a,3f12.3)')' Box sizes:            ',BOX
    write((stdout),*)'Sampled configurations:',NAV
    write((stdout),*)'Number of molecules of each species:'
    do I=1,NMType ! TODO simplify
        write((stdout),'(i4,i6,3x,a,f6.2)')I,NMolMType(I),NameMType(I)
    end do
    write((stdout),*)
    write((stdout),*)'Total MC steps:        ',MKSteps
    write((stdout),*)'MC steps with averaging',MKSteps-NMCStepsEquil
    write(fmt,'(i3)') NMType*3
    write(stdout,'(a,'//fmt//'a8)')&
                     ' Molecule Type         ',(NameMType(:))
    write((stdout),'(a,'//fmt//'f8.3)')&
                     ' MCStepAtom(MolType)   ',MCStepAtomMType(:)
    write((stdout),'(a,'//fmt//'f8.3)')&
                     ' MCTranStep(MolType)   ',MCTransStepMType(:)
    write((stdout),'(a,'//fmt//'f8.3)')&
                     ' MCRotStep(MolType)    ',MCRotStepMType(:)                     
    write((stdout),'(a,'//fmt//'f8.3)')&
                     'Acceptance ratio atoms ',dfloat(AccCntMType(:,1,1))/AccCntMType(:,2,1)
    write((stdout),'(a,'//fmt//'f8.3)')&
                     'Acceptance ratio trans ',dfloat(AccCntMType(:,1,2))/AccCntMType(:,2,2)
    write((stdout),'(a,'//fmt//'f8.3)')&
                     'Acceptance ratio rot   ',dfloat(AccCntMType(:,1,3))/AccCntMType(:,2,3)
    write((stdout),*)'Temperature            ',TEMP
    write((stdout),*)'Dielectric permittivity ',EPS
    write((stdout),*)'Electrostatic energy:  ',EELS*FKJM*FNR
    write((stdout),*)'Bond energy:           ',EINTS*FKJM*FNR
    write((stdout),*)'------------------------------------'
    write((stdout),*)'Average energy:        ',ENERS*FKJM*FNR
    PRES=NAtom*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*iOutFactor-((VIRij_frozen+VIRIij_frozen)/3.)*FPR
    write((stdout),*)'Pressure                ',PRES
    write((stdout),'(a,4f12.2)')'Pressure components    ',&
             -(VIRS*iOutFactor+VIRij_frozen)*FPR/3.0,-VIRE*iOutFactor*FPR/3.,-(VIIS*iOutFactor+VIRIij_frozen)*FPR/3.,NAtom*FPR
! Here I can add printing of RDF for non inverse run.
end subroutine PrintMCsummary    

subroutine MCStepAutoAdjust(iStep,psize,iTrans, iRot,MCStepAtomARMType,MCTransStepARMType, MCRotStepARMType,&
                    MCStepAtomMType, MCTransStepMType, MCRotStepMType,AccCntMType,NameMType, ID,stdout)
    use precision
    implicit none
    integer(8), intent(in):: AccCntMType(:,:,:)
    real(8), intent(inout):: MCStepAtomMType(:),MCTransStepMType(:), MCRotStepMType(:)
    real(8), intent(in):: MCStepAtomARMType(:),MCTransStepARMType(:), MCRotStepARMType(:)
    integer(4), intent(in):: psize, iTrans, iRot, ID, iStep
    integer(8), allocatable:: AccCNT_accum(:,:,:)
    real(8), allocatable:: AccRat_sampl(:,:)
    character(mol_name_len), intent(in):: NameMType(:)
    integer(4) NMType, MCSteps,i, ierror,stdout
    real(8), parameter:: a=0.4, b=0.4
    character(len=5) :: fmt !Auxilary variable for output format specification
#ifdef mpi
       include 'mpif.h'    
#endif        
! Collect Acceptance counters 
    NMType=size(MCStepAtomMType)
    if (psize.eq.0) then 
        MCSteps=iStep
    else
        MCSteps=iStep*psize
    endif
    allocate(AccCNT_accum(size(AccCntMType,1),size(AccCntMType,2),size(AccCntMType,3)))
    allocate(AccRat_sampl(size(AccCntMType,1),size(AccCntMType,3)))
    AccCNT_accum=0; AccRat_sampl=0
#ifdef mpi
    call MPI_AllREDUCE(AccCntMType,AccCnt_accum,size(AccCntMType),MPI_INTEGER8,MPI_SUM,MPI_COMM_WORLD,ierror)
!    write(*,*) 'AccCntMType',AccCntMType
!    write(*,*) 'AccCnt_accum',AccCnt_accum
#else
    AccCnt_accum=AccCntMType
    
#endif
! Calculate Acceptance Ratios    
    AccRat_sampl(:,1)=dfloat(AccCnt_accum(:,1,1))/dfloat(AccCnt_accum(:,2,1))
    AccRat_sampl(:,2)=dfloat(AccCnt_accum(:,1,2))/dfloat(AccCnt_accum(:,2,2))
    AccRat_sampl(:,3)=dfloat(AccCnt_accum(:,1,3))/dfloat(AccCnt_accum(:,2,3))
    if (ID.le.1) then
        write(6,*) 'MC step side automatic adjustment. MCStep:',iStep, 'NSteps=',MCSteps
        write(fmt,'(i3)') NMType*3
        write(6,'(a,'//fmt//'a8)')    ' Molecule Type         ',(NameMType(:))
        write((6),'(a,'//fmt//'f8.3)')' MCStepAtom(MolType)   ',MCStepAtomMType(:)
        write((6),'(a,'//fmt//'f8.3)')' MCTranStep(MolType)   ',MCTransStepMType(:)
        write((6),'(a,'//fmt//'f8.3)')' MCRotStep(MolType)    ',MCRotStepMType(:)                     
        write(6,*) 'Sampled Acceptance Ratio' 
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio atoms ',AccRat_sampl(:,1)
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio trans ',AccRat_sampl(:,2)
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio rot   ',AccRat_sampl(:,3)   
        write(6,*) 'Desired Acceptance Ratio' 
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio atoms ',MCStepAtomARMType(:)
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio trans ',MCTransStepARMType(:)
        write((6),'(a,'//fmt//'f8.3)')'Acceptance ratio rot   ',MCRotStepARMType(:)   
    endif
    ! Calculate update to MC step size
    do i=1,NMType
        MCStepAtomMType(i)=MCStepAtomMType(i)*(log(a*MCStepAtomARMType(i) +b))/(log(a*AccRat_sampl(i,1) +b))
        if (AccRat_sampl(i,2).gt.0) & ! to avoid error for ions, which do not have rotation and translation steps.
            MCTransStepMType(i)=MCTransStepMType(i)*(log(a*MCTransStepARMType(i) +b))/(log(a*AccRat_sampl(i,2) +b))
        if (AccRat_sampl(i,3).gt.0) &
            MCRotStepMType(i)=MCRotStepMType(i)*(log(a*MCRotStepARMType(i) +b))/(log(a*AccRat_sampl(i,3) +b))
    enddo
! Send updated values     
!#ifdef mpi    
!    call MPI_Bcast(MCStepAtomMType,size(MCStepAtomMType),MPI_REAL8,0,MPI_COMM_WORLD,ierror)
!    call MPI_Bcast(MCTransStepMType,size(MCTransStepMType),MPI_REAL8,0,MPI_COMM_WORLD,ierror)
!    call MPI_Bcast(MCRotStepMType,size(MCRotStepMType),MPI_REAL8,0,MPI_COMM_WORLD,ierror)
!#else
!    
!#endif    
! Terminate
    if (ID.le.1) then
        write(6,*) 'Adjusted MC steps'     
        write(6,'(a,'//fmt//'a8)')    ' Molecule Type         ',(NameMType(:))
        write((6),'(a,'//fmt//'f8.3)')' MCStepAtom(MolType)   ',MCStepAtomMType(:)
        write((6),'(a,'//fmt//'f8.3)')' MCTranStep(MolType)   ',MCTransStepMType(:)
        write((6),'(a,'//fmt//'f8.3)')' MCRotStep(MolType)    ',MCRotStepMType(:)                     
    endif             
    deallocate(AccCNT_accum,AccRat_sampl)
end subroutine
end
