! Module Monte-Carlo
! Here I will put all rewritten subroutines of MC engine and energy calculation
MODULE mcmod 
    use precision
    use strings
    use utilsmod
    use dynarray

    implicit none

    Type Angle
        integer(4):: Atom1, Atom2, Atom3 ! 1-2-3
        integer(4):: Bond
        real(8) :: phi, Eang
        logical:: LMove
    end type
    real(8),allocatable,private :: D_SIN(:),D_COS(:),sigmasin(:), sigmacos(:)    
    
    type (Angle), allocatable :: Angles(:) ! Array of angle-bonds present in the system
    integer(4):: NAngles ! How many angles are present in the system

    real(8),private, parameter :: RAD2DEG=180.d0/3.1415926535897932d0
    real(8), allocatable :: Rij_vec(:,:,:), Rij_vec_new(:,:), & ! vector of interparticle distances                            
                            Rij(:,:), Rij_new(:),& ! interparticle distance
                            Phi_new(:),&
                            RijM_vec_new(:,:,:), RijM_new(:,:), &
                            VIRij(:,:),VIRij_new(:), VIRijM_new(:,:),&
                            Ri_vec(:,:) ! Coordinates of the atoms not disturbed by PBC
    real(8), allocatable :: Eij(:,:),Eij_new(:),& ! NB energy term
                            Eelij(:,:), Eelij_new(:),& ! Electrostatic direct space energy term
                            Eang_new(:),&                           
                            EijM_new(:,:), EelijM_new(:,:)                            
    real(8)::               ENB_frozen, EB_frozen, EA_frozen, EelNB_frozen, EelBA_frozen, VIRij_frozen, VIRIij_frozen                          
    integer(4), allocatable:: NAngAtom(:), AngAtom(:,:) ! Arrays specifying angle-bonds (indexes in array angles) for each atom of the system.                     
    integer(4),allocatable:: NBondsAtom(:), BondNum(:,:), jAtomBond(:,:) ! How many bonds the atom is involved, the bond number, second atom involved in the bond
    integer(2), allocatable:: AdrGlobIndexAtomij(:,:)
    real(8) :: FKJM    
    real(8), private:: RDFINC    
    logical, allocatable:: MaskNB(:,:),MaskB(:,:),MaskA(:,:) ! Masks for the all-atom cross matrixes Rij, Eij, Eelij
    
    
    contains
    include "mcmod_MCstep_contributions.f90"
    include "mcmod_init.f90"
    include "mcmod_algorithm.f90"
    include "mcmod_average.f90"
    include "mcmod_ewald.f90"
    
   
subroutine PBC_vec(R,HBOX, BOX)
    implicit none
    real(8), intent(in) :: HBOX(3), BOX(3)
    real(8), intent(inout) :: R(:,:)
    integer(8) i

    forall (i=1:size(R,2))
            where (abs(R(:,i))>HBOX) R(:,i)=R(:,i)-sign(BOX,R(:,i))
    end forall
end subroutine PBC_vec
    
subroutine PBC_vecM(R,HBOX, BOX)
    implicit none
    real(8), intent(in) :: HBOX(3), BOX(3)
    real(8), intent(inout) :: R(:,:,:)
    integer(8) i,k

    forall (k=1:size(R,3))
        forall (i=1:size(R,2))
            where (abs(R(:,i,k))>HBOX) R(:,i,k)=R(:,i,k)-sign(BOX,R(:,i,k))
        end forall
    end forall
end subroutine PBC_vecM
    
subroutine MCPBC(R,BOX)
    implicit none
    real(8), intent(inout):: R(3)
    real(8), intent(in)::BOX(3)
    real(8) HBOX(3),CORSS    
!Multiple PBC realization
    HBOX=Box*0.5
#ifdef PBC_MULT
    do
      if(R(1).gt.-HBOX(1)) exit
      R(1)=R(1)+BOX(1)
    enddo
    do
    if(R(1).lt.HBOX(1)) exit
    R(1)=R(1)-BOX(1)
    enddo
    do
    if(R(2).gt.-HBOX(2)) exit
    R(2)=R(2)+BOX(2)
    enddo
    do
    if(R(2).lt.HBOX(2)) exit
    R(2)=R(2)-BOX(2)
    enddo
    do
    if(R(3).gt.-HBOX(3)) exit
    R(3)=R(3)+BOX(3)
    enddo
    do
    if(R(3).lt.HBOX(3)) exit
    R(3)=R(3)-BOX(3)
    enddo
#else
    if(R(1).gt. HBOX(1))R(1)=R(1)-BOX(1)
    if(R(1).lt.-HBOX(1))R(1)=R(1)+BOX(1)
    if(R(2).gt. HBOX(2))R(2)=R(2)-BOX(2)
    if(R(2).lt.-HBOX(2))R(2)=R(2)+BOX(2)
    if(R(3).gt. HBOX(3))R(3)=R(3)-BOX(3)
    if(R(3).lt.-HBOX(3))R(3)=R(3)+BOX(3)
#ifdef PBC_TO

    CORSS=HBOX(1)*int((4.0/3.0)*(abs(R(1))+abs(R(2))+abs(R(3)))/BOX(1))
    R(1)=R(1)-sign(CORSS,R(1))
    R(2)=R(2)-sign(CORSS,R(2))
    R(3)=R(3)-sign(CORSS,R(3))

#endif
#endif        
    return
end subroutine
    
subroutine EnergyForce(RR,IT,JT,NA,RMAX,PotNB_riSjS, Energ, Force)
    implicit none
    real(8), intent(IN) :: PotNB_riSjS(:,:,:), RR
    real(8), intent(in) :: RMAX       
    integer(4), intent(in):: IT,JT
    integer(4), intent(in):: NA
    real(8),intent(out):: ENERG, Force
    integer(4) NR1
    real(8) Al!, AL1

    NR1=int(RR/RDFINC+0.5) ! NR1=round(RR/RDFINC+0.5)
    if (NR1.le.0) NR1=1
    if (NR1.ge.NA) NR1=NR1-1        
    AL=(RR-(NR1-0.5)*RDFINC)/RDFINC
    ENERG=(1-AL)*PotNB_riSjS(NR1,IT,JT)+AL*PotNB_riSjS(NR1+1,IT, JT)
    FORCE=(PotNB_riSjS(NR1+1,IT, JT)-PotNB_riSjS(NR1,IT,JT))/RDFINC        
    return
end subroutine

subroutine EneriForci(RR,IPP,Rmin,Rmax,PotB_riBond,NPointsPotB,stdout, Eneri, Forci)
    real(8), intent(in):: RR,Rmin,Rmax,PotB_riBond(:,:)
    integer(4), intent(in):: IPP,stdout,NPointsPotB
    real(8), intent(out):: Eneri, FORCI
    real(8) RDINC,AL,RR1
    integer(4) NR1

    Eneri=0.0; FORCI=0.0
    RDINC=(Rmax-Rmin)/NPointsPotB
    if(RR.le.Rmin)then
        ENERI=PotB_riBond(1,IPP)
        write(stdout,'(a,2f10.4,i4)')' too small argument in Bond Energy calculation ENERI:',RR,Rmin,IPP
        return
    end if
    if(RR.gt.Rmax)then
        ENERI=PotB_riBond(NPointsPotB,IPP)
        write(stdout,'(a,2f10.4,i4)') ' too large argument in Bond Energy calculation ENERI:',RR,Rmax,IPP
        return
    end if
    NR1=(RR-Rmin)/RDINC+0.5
    if(NR1.le.0)NR1=NR1+1
    if(NR1.ge.NPointsPotB) NR1=NR1-1    
    RR1=Rmin+(NR1-0.5)*RDINC
    AL=(RR-RR1)/RDINC
    ENERI=(1-AL)*PotB_riBond(NR1,IPP)+AL*PotB_riBond(NR1+1,IPP)    
    Forci=(PotB_riBond(NR1+1,IPP)-PotB_riBond(NR1,IPP))/RDINC
    return
    end subroutine
subroutine Rotate2Mult(Rvec,phi,ax,ay,az)
    real(8), intent(inout):: Rvec(:,:)
    real(8), intent(in):: phi,ax,ay,az
    
    real(8) CosPhi,OneMinCos,SinPhi,x,y,z,xy,xz,yz,xx,yy,zz,RotMat(3,3)

    CosPhi=Dcos(phi)
    OneMinCos=1-CosPhi
    SinPhi=Dsin(phi)
    
    xy=ax*ay
    xz=ax*az
    yz=ay*az
    xx=ax*ax
    yy=ay*ay
    zz=az*az
    
    RotMat(1,1)= CosPhi+OneMinCos*xx
    RotMat(2,1)= OneMinCos*xy+SinPhi*az
    RotMat(3,1)= OneMinCos*xz - SinPhi*ay

    RotMat(1,2)= OneMinCos*xy - SinPhi*az
    RotMat(2,2)= CosPhi + OneMinCos*yy
    RotMat(3,2)= OneMinCos*yz + SinPhi*ax

    RotMat(1,3)= OneMinCos*xz + SinPhi*ay
    RotMat(2,3)= OneMinCos*yz - SinPhi*ax
    RotMat(3,3)= CosPhi + OneMinCos*zz

    Rvec=MatMul(RotMat,Rvec)
    return
end subroutine

!====================== MCGETCOM ======================================
! Calculate center of mass of the given molecule I. Update relative coordinates of the molecule's atoms
subroutine MCgetCOM(iMol, MTypeMol,MassSite,SumMassMType,&
            Wvec,BOX,AdrAtomMol,SiteAtom,XvecCOMMol)
    integer(4), intent(in):: MTypeMol(:),AdrAtomMol(:), SiteAtom(:)!,AdrSiteMType(:),NSiteMType(:)
    integer(4),intent(in):: iMol
    real(8), intent(in):: MassSite(:),SumMassMType(:)
    real(8), intent(inout):: XvecCOMMol(:,:),Wvec(:,:)
    real(8), intent(in):: BOX(3)
!----------Local Variables----------------------------------------------		    
    real(8) Rcom(3), dR(3), HBOX(3),dx(3)
    integer AtomFirst, AtomLast, iAtom
    HBOX=BOX*0.5
    AtomFirst=AdrAtomMol(iMol)+1
    AtomLast=AdrAtomMol(iMol+1)
    ! Calculate COM in absolute coordinates
    Rcom=sum(Ri_vec(1:3,AtomFirst:AtomLast)*spread(MassSite(SiteAtom(AtomFirst):SiteAtom(AtomLast)),1,3),dim=2)/&
        SumMassMType(MTypeMol(iMol))
    ! Move COM in the origin box if necessary
    dX=RCOM
    call MCPBC(dX,BOX)
    dR=dX-Rcom ! Vector to shift atoms of the molecule     
    Rcom=dX
    ! Shift atoms of the molecule
    Ri_vec(:,AtomFirst:AtomLast)=Ri_vec(:,AtomFirst:AtomLast)+spread(dR,2,AtomLast-AtomFirst+1)
    ! Calculate distances relative to COM
    forall (iAtom=AtomFirst:AtomLast)  ! This cycle is not required, as it shall produce the same values as they were initially.                    
    ! But it is kept for consistency check
        Wvec(:,iAtom)=Ri_vec(:,iAtom)-Rcom
    end forall 
    XvecCOMMol(:,imol)=Rcom
    return
end subroutine
!!==========  ===============================================
! Initialize Fourier - former INIFUR
subroutine MCmodInitEwald(SSIN,SCOS,BOX,VOL,ALPHA,AF,FQ,RECUT,&
                        COULF,Recvec, RKV,ESF,FKJM,NKV,NAtom,ChargeAtom,stdout)
    implicit none
    real(8), intent(in):: BOX(3),ChargeAtom(:),VOL,FKJM, COULF, AF, FQ, RECUT
    real(8), allocatable, intent(out):: Recvec(:,:),RKV(:)
    real(8), allocatable, intent(out):: SSIN(:),SCOS(:)
    real(8), intent(out):: ESF, ALPHA
    integer(4), intent(in):: NAtom,stdout
    integer(4), intent(out):: NKV
    include "dimpar.h"

    real(8) CUTK,CUTK2,FK(3),RK2,AFF
    integer(4) KMAX(3),IKV,IKZ,IKX,IKY,KY1,KY2,I
    
    ALPHA=AF/RECUT
    CUTK2=4.d0*ALPHA**2*FQ
    CUTK=sqrt(CUTK2)
    FK=2.d0*PI/BOX
    KMAX=CUTK/FK+1
    IKV=0
    do IKZ=-KMAX(3),KMAX(3)
        do IKX=0,KMAX(1)
            if(IKX.eq.0)then
                if(IKZ.gt.0)then
                    KY1=0
                else
                    KY1=1
                end if
                KY2=KMAX(2)
            else
                KY1=-KMAX(2)
                KY2=KMAX(2)
            end if
            do IKY=KY1,KY2
                RK2=sum((FK**2)*([IKX,IKY,IKZ]**2))
                if(RK2.le.CUTK2)then
#ifdef PBC_TO
                    if (abs(mod(IKX+IKY+IKZ,2)).eq.0) IKV=IKV+1 ! For truncated octahedron we consider only Kvec with even sum of coordinates, odd will be zero
#else
                    IKV=IKV+1
#endif
                end if
            end do
        end do
    end do
    if (.not. allocated(Recvec)) then 
       allocate(RKV(IKV),Recvec(3,ikv))
       recvec=0.d0
       RKV(:)=0.d0
    endif
    IKV=0
    do IKZ=-KMAX(3),KMAX(3)
        do IKX=0,KMAX(1)
            if(IKX.eq.0)then
                if(IKZ.gt.0)then
                    KY1=0
                else
                    KY1=1
                end if
                KY2=KMAX(2)
            else
                KY1=-KMAX(2)
                KY2=KMAX(2)
            end if
            do IKY=KY1,KY2
                RK2=sum((FK**2)*([IKX,IKY,IKZ]**2))
#ifdef PBC_TO
                if((RK2.le.CUTK2).and.(abs(mod(IKX+IKY+IKZ,2).eq.0)))then
#else
                if(RK2.le.CUTK2)then
#endif
                    IKV=IKV+1
                    Recvec(:,IKV)=FK*[IKX,IKY,IKZ]
                    RKV(IKV)=4.*PI**2*COULF/(VOL*PI)*dexp(-0.25*RK2/ALPHA**2)/RK2
                end if
            end do
        end do
    end do
    NKV=IKV
    write((stdout),*)'Kmax=',KMAX,'    Num. of k-vectors ',NKV
    if (.not. allocated(SSIN)) allocate(SSIN(NKV))
    if (.not. allocated(SCOS)) allocate(SCOS(NKV))
    if (.not. allocated(D_SIN)) allocate(D_SIN(NKV))
    if (.not. allocated(D_COS)) allocate(D_COS(NKV))
    if (.not. allocated(sigmasin)) allocate(sigmasin(NKV))
    if (.not. allocated(sigmacos)) allocate(sigmacos(NKV))
!    allocate(SSIN(NKV),SCOS(NKV),D_SIN(NKV),D_COS(NKV),sigmasin(NKV),sigmacos(NKV))
    SSIN(:)=0.d0; SCOS(:)=0.d0; D_SIN(:)=0.d0; D_COS(:)=0.d0; sigmasin(:)=0.d0; sigmacos(:)=0.d0
!    endif       
    ESF=-sum(ChargeAtom(1:NAtom)**2)*ALPHA*COULF/sqrt(PI)
    write((stdout),*)' el. self-energy ',ESF*FKJM
    return
end subroutine

END
