subroutine MCmodNewStep(NAtom, iAtom, deltaR, RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom, STypeAtom, BOX, NA, POTNB_riSjS,&
    MolAtom, NPointsPotB, RMaxPotB, RMinPotB, PotB_riBond, stdout, Lexit)
    implicit none
    integer(4), intent(in):: NAtom, iAtom, STypeAtom(:), NA, MolAtom(:), NPointsPotB(:),stdout
    real(8), intent(in):: deltaR(3), RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom(:), BOX(3), POTNB_riSjS(:,:,:),&
                RMaxPotB(:), RMinPotB(:),PotB_riBond(:,:)                  

    !Local
    integer(4) jatom,i
    real(8) Force
    logical Lexit

    ! New distances:         
    Rij_vec_new=Rij_vec(:,:,iAtom)-spread(deltaR,2,NAtom)        
    Rij_vec_new(:,iAtom)=0.0
    VIRij_new(:)=0.0
    ! Apply PBC for the new vector distances
    call PBC_vec(Rij_vec_new,BOX*0.5, BOX)            
    ! Calculate distance modulus
    Rij_new=sqrt(sum(Rij_vec_new**2,dim=1))    
    !Set all new energies to zero
    Eij_new=0.0; Eelij_new=0.0       
    ! Calculate Pairwise-Bond energy terms
    call MCmodNewStepB(iAtom,RMaxPotB, RMinPotB, PotB_riBond, NPointsPotB, stdout,LExit)
    if (lExit) return 
    ! Calculate Angle-bending energy terms
    call MCmodNewStepA(iAtom, RMaxPotB, RMinPotB, PotB_riBond, NPointsPotB, stdout,LExit)
    if (lExit) return
    ! Calculate NB energy terms, where MaskNB and r<rcut
    do jatom=1, NAtom
        if (MaskNB(jAtom,iAtom)) then            
            if (Rij_new(jAtom)<RCUT) then
                call EnergyForce(Rij_new(jAtom),STypeAtom(jAtom),STypeAtom(iAtom),NA,RMAX,PotNB_riSjS, Eij_new(jAtom), Force)                
                VIRij_new(jAtom)=Force*Rij_new(jAtom)
            endif
        endif
    enddo       
    ! Calculate electrostatic energy terms or corrections (for bonded atoms)
    if (ChargeAtom(iAtom).ne.0.0) then
        do jAtom=1, NAtom
            if ((jAtom.ne.iAtom).and.(ChargeAtom(jAtom).ne.0.0)) then          
                if (MaskNB(jAtom,iAtom).and.(Rij_new(jAtom)<RECUT)) then
                    Eelij_new(jAtom)=ChargeAtom(iAtom)*ChargeAtom(jAtom)*COULF*derfc(ALPHA*Rij_new(jAtom))/Rij_new(jAtom)
                endif
                if (MaskB(jAtom,iAtom).or.MaskA(jAtom,iAtom)) then ! Bonded
                    Eelij_new(jAtom)=ChargeAtom(iAtom)*ChargeAtom(jAtom)*COULF*((derfc(ALPHA*Rij_new(jAtom))-1.)/Rij_new(jAtom))                    
                endif
            endif
        enddo
    endif
end subroutine
    
subroutine MCmodNewStepB(iAtom, RMaxPotB, RMinPotB, PotB_riBond, NPointsPotB, stdout,LExit)
    !Global
    implicit none
    integer(4), intent(in):: iAtom, NPointsPotB(:), stdout
    real(8), intent(in):: RMaxPotB(:), RMinPotB(:),PotB_riBond(:,:)       
    logical, intent(out):: LExit        
    !Local
    integer(4) j, N
    real(8) Force

    lExit=.false.
    N=NBondsAtom(iAtom)
    if (N.gt.0) then         
        if (ANY(Rij_new(jAtomBond(1:N,iAtom)).gt.RMaxPotB(BondNum(1:N,iAtom)))) then
            LExit=.true.; return
        endif
        if (ANY(Rij_new(jAtomBond(1:N,iAtom)).lt.RMinPotB(BondNum(1:N,iAtom)))) then
            LExit=.true.; return
        endif
        ! Calculate pairwise Bond energy terms
        do j=1,N 
            call EneriForci(Rij_new(jAtomBond(j,iAtom)),BondNum(j,iAtom),RMinPotB(BondNum(j,iAtom)),&
                RMaxPotB(BondNum(j,iAtom)),PotB_riBond,NPointsPotB(BondNum(j,iAtom)),stdout, Eij_new(jAtomBond(j,iAtom)), Force)                                    
            VIRij_new(jAtomBond(j,iAtom))=Force*Rij_new(jAtomBond(j,iAtom))
        enddo
    endif
end subroutine
    
subroutine MCmodNewStepA(iAtom, RMaxPotB, RMinPotB, PotB_riBond, NPointsPotB, stdout,LExit)
    !Global
    implicit none
    integer(4), intent(in):: iAtom, NPointsPotB(:), stdout
    real(8), intent(in):: RMaxPotB(:), RMinPotB(:)
    real(8), intent(in):: PotB_riBond(:,:)       
    logical, intent(out):: LExit        
    !Local
    integer(4) i,j, N
    type (angle) iAngle
    real(8) Energ, Force

    LExit=.false.
    N=NAngAtom(iAtom) ! Number of A-bonds the atom involved in as a side atom
    Eang_new=0.0    
    ! Calculate new angles related to the shifted atom iAtom
    do i=1, NAngAtom(iAtom)
        iAngle=Angles(AngAtom(i,iAtom))
        if (iAtom.eq.iAngle%Atom1) then !First atom in the triplet
            Phi_new(i)=dacos(dot_product(Rij_vec_new(:,iAngle%Atom2),Rij_vec(:,iAngle%Atom2,iAngle%Atom3))/&
                                    (Rij_new(iAngle%Atom2)*Rij(iAngle%Atom2,iAngle%Atom3)))*RAD2DEG
        elseif (iAtom.eq.iAngle%Atom2) then ! Second atom in the triplet
            Phi_new(i)=dacos(dot_product(Rij_vec_new(:,iAngle%Atom1),Rij_vec_new(:,iAngle%Atom3))/&
                                    (Rij_new(iAngle%Atom1)*Rij_new(iAngle%Atom3)))*RAD2DEG            
        else! third atom in the triplet 
            Phi_new(i)=dacos(dot_product(Rij_vec_new(:,iAngle%Atom2),Rij_vec(:,iAngle%Atom2,iAngle%Atom1))/&
                                    (Rij_new(iAngle%Atom2)*Rij(iAngle%Atom2,iAngle%Atom1)))*RAD2DEG            
        endif 
    enddo
    ! Check if the new values are within the allowed range of angles
    if (ANY(Phi_new(1:N).gt.RMaxPotB(Angles(AngAtom(1:N,iAtom))%Bond))) then
            LExit=.true.; return
    endif
    if (ANY(Phi_new(1:N).lt.RMinPotB(Angles(AngAtom(1:N,iAtom))%Bond))) then
        LExit=.true.; return
    endif
    ! Calculate new angle-bending energy terms
    do j=1,N 
        iAngle=Angles(AngAtom(j,iAtom))
        call EneriForci(Phi_new(j),iAngle%Bond, RMinPotB(iAngle%Bond),&
            RMaxPotB(iAngle%Bond),PotB_riBond,NPointsPotB(iAngle%Bond),stdout, Energ, Force)
        Eang_new(j)=Energ
    enddo    
end subroutine
    
subroutine MCmodNewTrans(NAtom, iMol, deltaR, RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom, STypeAtom, BOX, NA, POTNB_riSjS,&
    NSiteMType, MTypeMol,AdrAtomMol, stdout)
    implicit none
    integer(4), intent(in):: NAtom, iMol, STypeAtom(:), NA, MTypeMol(:), NSiteMType(:),AdrAtomMol(:), stdout
    real(8), intent(in):: deltaR(3), RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom(:), BOX(3), POTNB_riSjS(:,:,:)
    !Local
    integer(4) NAtomsInMol, iAtom, jAtom,i
    real(8) Force

    ! New distances:    
    NAtomsInMol=NSiteMType(MTypeMol(iMol))
    RijM_vec_new(1:3,1:NAtom,1:NAtomsinMol)=Rij_vec(:,:,AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))-&
                                            spread(spread(deltaR,2,NAtom),3,NAtomsInMol)        
    RijM_vec_new(:,AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
            Rij_vec(:,AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    ! Apply PBC for the new vector distances
    call PBC_vecM(RijM_vec_new(1:3,:,1:NAtomsInMol),BOX*0.5, BOX)            
    
    ! Calculate distance modulus
    RijM_new=sqrt(sum(RijM_vec_new**2,dim=1))    
    !Set all new energies and virial parts to zero
    EijM_new=0.0; EelijM_new=0.0        
    EijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
        Eij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    EelijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
        Eelij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    VIRijM_new(:,:)=0.0
    VIRijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
            VIRij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    ! Calculate NB energy terms, where MaskNB and r<rcut
    do i=1, NAtomsInMol
        iAtom=AdrAtomMol(iMol)+i
        do jatom=1, AdrAtomMol(iMol)
            if (MaskNB(jAtom,iAtom)) then
                if (RijM_new(jAtom,i)<RCUT) then
                   call EnergyForce(RijM_new(jAtom,i),STypeAtom(jAtom),STypeAtom(iAtom),NA,RMAX,PotNB_riSjS,EijM_new(jAtom,i),Force)
                   VIRijM_new(jAtom,i)=Force*RijM_new(jAtom,i)
                endif
            endif
            if ((ChargeAtom(jAtom).ne.0.0).and.(RijM_new(jAtom,i)<RECUT)) then          
                if (MaskNB(jAtom,iAtom)) EelijM_new(jAtom,i)=ChargeAtom(iAtom)*ChargeAtom(jAtom)*COULF*&
                                derfc(ALPHA*RijM_new(jAtom,i))/RijM_new(jAtom,i)
            endif
        enddo       
        do jatom=AdrAtomMol(iMol+1)+1, NAtom
            if (MaskNB(jAtom,iAtom)) then
                if (RijM_new(jAtom,i)<RCUT) then
                   call EnergyForce(RijM_new(jAtom,i),STypeAtom(jAtom),STypeAtom(iAtom),NA,RMAX,PotNB_riSjS,EijM_new(jAtom,i),Force)
                   VIRijM_new(jAtom,i)=Force*RijM_new(jAtom,i)
                endif
            endif
            if ((ChargeAtom(jAtom).ne.0.0).and.(RijM_new(jAtom,i)<RECUT)) then          
                if (MaskNB(jAtom,iAtom)) EelijM_new(jAtom,i)=ChargeAtom(iAtom)*ChargeAtom(jAtom)*COULF*&
                                derfc(ALPHA*RijM_new(jAtom,i))/RijM_new(jAtom,i)
            endif
        enddo       
    enddo
end subroutine

subroutine MCmodNewRot(NAtom, iMol, deltar,phi, RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom, STypeAtom, BOX, NA, POTNB_riSjS,&
    NSiteMType, MTypeMol,AdrAtomMol, SumMassMType, MassSite, SiteAtom, stdout)
    implicit none
    integer(4), intent(in):: NAtom, iMol, STypeAtom(:), NA, MTypeMol(:), NSiteMType(:),AdrAtomMol(:),SiteAtom(:), stdout
    real(8), intent(in):: deltaR(3),phi, RCUT,RECUT, RMAX, ALPHA, COULF, ChargeAtom(:),BOX(3), POTNB_riSjS(:,:,:), SumMassMType(:),&
        MassSite(:)                
    !Local
    integer(4) NAtomsInMol, iAtom, jAtom,i, AtomFirst, AtomLast, j
    real(8) Rcom(3), Rcom_new(3),dR(3), Force
    real(8), allocatable:: RicomOld(:,:),RicomNew(:,:)

    ! Atoms in the molecule    
    NAtomsInMol=NSiteMType(MTypeMol(iMol))
    AtomFirst=AdrAtomMol(iMol)+1
    AtomLast=AdrAtomMol(iMol+1)
    ! Calculate COM of the molecule
    Rcom=sum(Ri_vec(1:3,AtomFirst:AtomLast)*spread(MassSite(SiteAtom(AtomFirst):SiteAtom(AtomLast)),1,3),dim=2)/&
        SumMassMType(MTypeMol(iMol))
   Rcom_new=Rcom
    ! Shift it to the origin box
    call MCPBC(Rcom_new, BOX)
    dR=Rcom_new-Rcom
    Ri_vec(:,AtomFirst:AtomLast)=Ri_vec(:,AtomFirst:AtomLast)+spread(dR,2,NAtomsInMol)
    ! Calculate The new positions of the atoms after rotation with respect to COM
    allocate(RicomOld(3,NAtomsInMol))
    allocate(RicomNew(3,NAtomsInMol))
    RicomOld(1:3,1:NAtomsInMol)=Ri_vec(1:3,AtomFirst:AtomLast)-spread(Rcom_new,2,NAtomsInMol)
    RicomNew=RicomOld    
    call Rotate2Mult(RiCOMNew,phi,deltar(1),deltar(2),deltar(3))
    ! New distances in the system
    RijM_vec_new(1:3,1:NAtom,1:NAtomsInMol)=Rij_vec(1:3,1:NAtom,AtomFirst:AtomLast)-&
        (spread(RiCOMNew(1:3,1:NAtomsInMol)-RiCOMOld(1:3,1:NAtomsInMol),2,NAtom))
    do i=1, NAtomsInMol
        do j=1, NAtomsInMol
            RijM_vec_new(1:3,j+AtomFirst-1,i)=RiCOMNew(1:3,j)-RiCOMNew(1:3,i)
        enddo
    enddo    
    ! Apply PBC for the new vector distances
    call PBC_vecM(RijM_vec_new(1:3,1:NAtom,1:NAtomsInMol),BOX*0.5, BOX)                
    ! Calculate distance modulus
    RijM_new=sqrt(sum(RijM_vec_new**2,dim=1))   
    !Set all new energies to zero
    EijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
         Eij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    EelijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
         Eelij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    VIRijM_new(:,:)=0.0
    VIRijM_new(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),1:NAtomsInMol)=&
         VIRij(AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1),AdrAtomMol(iMol)+1:AdrAtomMol(iMol+1))
    ! Calculate NB energy terms, where MaskNB and r<rcut
    do i=1, NAtomsInMol
        iAtom=AdrAtomMol(iMol)+i
	do jatom=1, NAtom
            if (MaskNB(jAtom,iAtom)) then
		EijM_new(jAtom,i)=0.0
		EelijM_new(jAtom,i)=0.0
                if (RijM_new(jAtom,i)<RCUT) then
                   call EnergyForce(RijM_new(jAtom,i),STypeAtom(jAtom),STypeAtom(iAtom),NA,RMAX,PotNB_riSjS,EijM_new(jAtom,i),Force)                
                   VIRijM_new(jAtom,i)=Force*RijM_new(jAtom,i)                   
                endif
            	if ((ChargeAtom(jAtom).ne.0.0).and.(RijM_new(jAtom,i)<RECUT)) then          
                	EelijM_new(jAtom,i)=ChargeAtom(iAtom)*ChargeAtom(jAtom)*COULF*&
                                derfc(ALPHA*RijM_new(jAtom,i))/RijM_new(jAtom,i)
	        endif
            endif
        enddo       
    enddo
end subroutine
