!========================= DIFEW ================================
!
!  Change of reciprocal Ewald energy if iAtom-th particle change
!  position from Rold to Rnew
!
subroutine MCmodDifEw(Rnew,CHI,DEW,Recvec,RKV,SSIN,SCOS,NKV,Rold)
! Global variables
    implicit none
    real(8), intent(in):: Rnew(3),CHI,Recvec(:,:),RKV(:),SSIN(:),SCOS(:),Rold(3)
    real(8), intent(out):: DEW

    integer(4), intent(in):: NKV
    ! Local variables
    real(8) SCO,SCN,SCP,SINR,DSN,DCS,DDD, sine,cosine
    integer(4) IKV
    !   reciprocal space Evald
    DEW=0.d0
    do IKV=1,NKV      
        SCO=0.5*dot_product(Recvec(:,IKV),Rold)
        SCN=0.5*dot_product(Recvec(:,IKV),Rnew)
        SCP=SCO+SCN
        SINR=sin(SCN-SCO)*CHI*2.d0
        call sincos(SCP,sine,cosine)        
!        sine=sin(SCP)
!        cosine=sqrt(1.d0-sine*sine)
        DSN=cosine*SINR
        DCS=-sine*SINR
        D_SIN(IKV)=DSN
        D_COS(IKV)=DCS
        DDD=((2.d0*SSIN(IKV)+DSN)*DSN+(2.d0*SCOS(IKV)+DCS)*DCS)*RKV(IKV)
        DEW=DEW+DDD
    end do
    return
end  subroutine MCmodDIFEW

!========================= DIFEWM ================================
!
!  Change of reciprocal Ewald energy if "I" molecule, represented by list of charged atoms ichatomlist change
!  position
!
subroutine MCmodDIFEWM(AdrAtomMol,iMol,Recvec,RKV,SSIN,SCOS,Svec,NKV,RN,ChargeAtom,DEW)
!Global variables
    implicit none
    integer(4), intent(in):: AdrAtomMol(:), iMol
    integer(4), intent(in):: NKV ! Number of charged shifted atoms
    real(8), intent(out):: DEW ! Change in reciprocal Ewald energy because of shift
    real(8),intent(in) :: Recvec(:,:),RKV(:),SSIN(:),SCOS(:),Svec(:,:),ChargeAtom(:),RN(:,:) ! new trial coordinates of charged shifted atoms    
! local variables
    integer i,inkv,II
    real(8):: dp_old, dp_new,sin_dp_old,cos_dp_old,sin_dp_new,cos_dp_new

    do inkv=1,NKV ! over all K-vectors
        sigmasin(inkv)=0.d0 ! Check if this line can be removed later
	sigmacos(inkv)=0.d0 ! Check if this line can be removed later
	do i=1,AdrAtomMol(iMol+1)-AdrAtomMol(iMol) ! Over all atoms in the molecule iMol
            II=AdrAtomMol(iMol)+i ! Global atom number of i-th charged atom
            if (ChargeAtom(II).ne.0.0) then! If atoms are charged                
                dp_old=dot_product(Recvec(:,inkv),Svec(:,II))
                dp_new=dot_product(Recvec(:,inkv),RN(:,i))
                call  sincos(dp_old,sin_dp_old,cos_dp_old)
                call  sincos(dp_new,sin_dp_new,cos_dp_new)
                sigmacos(inkv)=sigmacos(inkv)+ChargeAtom(II)*(cos_dp_new-cos_dp_old)
                sigmasin(inkv)=sigmasin(inkv)+ChargeAtom(II)*(sin_dp_new-sin_dp_old)
            endif
       	enddo
!	deltae=deltae+(sigmacos(inkv)*(2.0*SCOS(inkv)+sigmacos(inkv))+sigmasin(inkv)*(2.0*SSIN(inkv)+sigmasin(inkv)))*RKV(inkv)
    enddo
    DEW=sum((sigmacos*(2.0*SCOS+sigmacos)+sigmasin*(2.0*SSIN+sigmasin))*RKV)
end subroutine MCmodDIFEWM!end subroutine RECSIN

!-------------------------------------------------------------------------------------------------------------------------
subroutine MCmodEwald(EFUR,Recvec,Svec,ChargeAtom,SSIN,SCOS,RKV,&
				NAtomChrgAtom,AtomChrgAtom, NKV)
    implicit none                                
    integer(4),intent(in):: NKV,NAtomChrgAtom,AtomChrgAtom(:)
    real(8), intent(out):: EFUR,SSIN(:),SCOS(:)
    real(8), intent(in):: Recvec(:,:),RKV(:),Svec(:,:),ChargeAtom(:)    
    ! Local variables  
    integer(4) :: IKV,IA,I
    real(8) SCS,SSN
    real(8) sin_SC,cos_SC

!   reciprocal space Evald
    do IKV=1,NKV
        SCS=0.
        SSN=0.
        do IA=1,NAtomChrgAtom
            I = AtomChrgAtom(IA)
            call sincos(dot_product(Recvec(:,IKV),Svec(:,I)),sin_SC,cos_SC)
            SSN=SSN+ChargeAtom(I)*sin_SC
            SCS=SCS+ChargeAtom(I)*cos_SC
        end do        
	SSIN(IKV)=SSN
	SCOS(IKV)=SCS
    end do
    EFUR=sum(RKV*(SSIN**2+SCOS**2))
    return
end subroutine

