program MAGIC
!*
!*  MagiC's Monte Carlo Engine - direct and inverse simulation
!*
!*  Construction of effective potentials for arbitrary molecular
!*  systems from given RDF
!*
!*  Units: distances A
!*         Energy - kJ/mol
!*
!*  Alexander Lyubartsev      sasha@physc.su.se
!*  Alexander Mirzoev alexander.mirzoev@mmk.su.se or amirzoev@ntu.edu.sg
!*
!*  The main module
!*
!DEC$ FREEFORM
use precision
use dynarray
use utilsmod
use timer
use iomod 
use imcmod
use mcmod
use magicmod
implicit none

#ifdef mpi
include 'mpif.h'
#endif
include "molsim.h"

! Initialization
call InitMagic(psize, rank, ID)
! Here all input files are read, and default parameters are set      
call ReadAll(&
! ReadInput 
    NMType, NameMType, NMolMType, NA, LMoveMType, LRDF,LPOT, NMCSteps, NMCStepsEquil, MCStepAtomMType, &
    MCTransStepMType, MCRotStepMType, ITRANS, IROT,&
    RECUT, NRS, POTCUT,AF, FQ, IOUT,&
    IAV, REGP, DPOTM, LCRD, LCRDPASS, RTM, IREPT, LIMC, IPOTCORRCHECK, LPotCorrCheck,PotCorrCheckLevel, &
    LEV, BaseOutFileName, FSTART,NFStart,FCRD, LXMOL, ITR, NMCAutoAdjust, MCStepAtomARMType,MCTransStepARMType, MCRotStepARMType, &
    VOL, BOX, FRT, TEMP, FKJM, FPR, COULF, EPS, IAV, &
!               
    NSiteMType, NBondMType, SumMassMType, NSite,NameSite,MTypeSite, RSite, MassSite, ChargeSite,NSType, MTypeSType, NameSType,&
    NBond, NBondB, NBondA, Bond_iSjS, iSiteinBond, jSiteinBond, kSiteinBond, NSiteGroupsInBond, BTypeBond,MTypeBond, &
! Set references
    NMol,MTypeMol,XvecCOMMol, NAtom, MolAtom, STypeAtom, SiteAtom, ChargeAtom, NAtomChrgAtom, AtomChrgAtom, &
    AdrAtomMol, AdrMolMType, AdrSiteMType, AdrAtomMType, AdrBondMType, NPairsTotalBond, NMIN, Svec,Wvec,&
! Setreferences-SetAngleBonds                   
    SitesInGroupBondMType, SideSiteABond, NABondSite, SiteABondMType,EdgeSiteABond_iSjS,&
! Read RDF               
    FILRDF, StateNB_iSjS, StateB_iBond,IndexGlobNB_riSjS, IndexGlobB_riBond, RMAX,RMIN,RDFref_NB_riSjS,RDFref_B_riBond, &
! Read Pot
    FilPot,PotNB_riSjS, PotB_riBond,NPointsPotB, RMaxPotB, RMinPotB,&
! Initialize
    NactNB,IND, INDI, CORP,CORS,npairsrdf,AdrGlobIndexAtomij,RCUT, RDFINC, stdout,iPrint,rank, ID, psize)

! Initialize Monte Carlo engine variables
call MCmodInit(NA,RMAX,NMType, NAtom, NBond, NMolMType, NBondMType,MTypeBond,NSiteMType,SiteAtom, Bond_iSjS, MolAtom, BTypeBond,&
        AdrAtomMol,AdrBondMType, AdrSiteMType,&
        iSiteInBond,jSiteInBond,kSiteInBond,NSiteGroupsInBond,StateNB_iSjS,StateB_iBond,STypeAtom, AccCntMType,&
        iPrint,stdout)
call TimerStamp(stdout,'Initialization finished:')        
do ! IMC loop -----------------------------------------------------------------------------------------------
    !  Setting zero-th for intramolec. potential
    call ShiftIntraPotToZero(NBond,NPointsPotB,NMIN,PotB_riBond)
#ifdef mpi
    if (ID.gt.0) then 
        call TimerStart(T_MPI)
        call MPI_Bcast(NMIN,size(NMIN),MPI_INTEGER,0,MPI_COMM_WORLD,ierror) ! Sending NMIN from master-process to all other
        call TimerStop(T_MPI)
    endif
#endif
    call TimerStart(T_IMCinit)
    call ResetCounters(stdout, NAV, AccCntMType, ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES, CORS, CORP,npairsrdf)     
!!  Preparing initial state geometry
    call InitGeometry(LCRD, FSTART,NFStart, FCRD, LCRDPASS, LMoveMType, ID, stdout, iPrint, iREP, NAtom,&
        NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,NameMType,&
        XvecCOMMol, Svec, Wvec, RSite,BOX, Ri_vec)
    call MCmodInitEwald(SSIN,SCOS,BOX,VOL,ALPHA,AF,FQ,RECUT,&
                        COULF,Recvec, RKV,ESF,FKJM,NKV,NAtom,ChargeAtom,stdout)        
    call FrozenEnergy(NAtom, NA,  StateNB_iSjS, StateB_iBond, STypeAtom, SiteAtom, Bond_iSjS,MolAtom,&
                        NBondsAtom,BondNum, jAtomBond, NPointsPotB, RCUT,RMAX, RECUT,BOX, ChargeAtom, &
                        RMinPotB, RMaxPotB,PotNB_riSjS,PotB_riBond, COULF, ALPHA, stdout )
    call MCmodConstruct(NA, RMAX, RCUT,RECUT, NAtom, NMol, BOX, COULF, ALPHA,&
        Svec,Wvec,XvecCOMMol, STypeAtom, PotNB_riSjS, ChargeAtom, SiteAtom, MolAtom, &
        IND, RMinPotB, RMaxPotB, NPointsPotB,&
        PotB_riBond, AdrAtomMol,MTypeMol, MassSite,SumMassMType, iPrint,stdout, istep.gt.NMCStepsEquil, 0) 
    call TimerStart(T_Ewald)
    call MCmodEwald(EFUR,Recvec,Svec,ChargeAtom,SSIN,SCOS,RKV,NAtomChrgAtom,AtomChrgAtom, NKV)                                        
    call TimerStop(T_Ewald)
    call MCEnergy(ENER, EINT, EINT_ang, EEL, EFUR,ESF, 1, stdout) ! Set iPrint to 1 to suppress Error in Energy message
    write((stdout),*)' Init. energy = ',ENER*FKJM,' fourier ',EFUR*FKJM
    if(ITR.gt.0) then ! If writing trajectory: Open trajectory output file and write first snapshot
        call OpenTraj(ID,psize,IREP,BaseOutFileName)
        call WriteTraj(psize,0,NAtom,NameSite,SiteAtom,Ri_vec,BOX)
    endif        
    call TimerStop(T_IMCinit)
!Here we are done with all the initialization  !!!
    
!   Start simulation
    write((stdout),*) '' 
    write((stdout),*) '' 
    write((stdout),*)'==================================================='
    write((stdout),*)'About to start ',NMCSteps,' Monte Carlo steps; iteration',IREP
    write((stdout),*)'==================================================='
    write((stdout),*)' Total atoms      ',NAtom
    write((stdout),*)' Total molecules  ',NMol
    write((stdout),*)' Interaction sites',NSType
    write((stdout),*)' Atom sites       ',NSite
    write((stdout),*)' Molecule types   ',NMType
    write((stdout),*)'==================================================='
    call TimerStamp(stdout,'')
    if(IPRINT.ge.6)then
        write((stdout),*)' array Bond_iSjS'
        do I=1,NSite
            write((stdout),*)(Bond_iSjS(I,J),J=1,NSite)
        end do
    end if
    call flush(stdout)
    do iStep=1, NMCSteps ! ===================MC simulations Loop=======================================================================                           
        if (IPRINT.gt.7)   write ((stdout),*) 'AccRat:',AccCntMType(:,1,:),'ISTEP:',ISTEP
    ! Single atom displacement MC step
        call TimerStart(T_MCStep)
        call MCmodStep( NAtom, NA, MTypeMol,MolAtom, STypeAtom, NPointsPotB,&
             NKV, iPrint, stdout,LMoveMType,MCStepAtomMType, ChargeAtom, BOX, RCUT, RECUT, RMAX, ALPHA, COULF,&
            PotNB_riSjS, PotB_riBond, RMaxPotB, RMinPotB,SSIN, SCOS,ENER,EEL,EINT,EINT_ang,Svec,Recvec,RKV,AccCntMType(:,:,1))            
        call TimerStop(T_MCStep)
    ! Molecule translation MC step
        if ((ITRANS.gt.0).and.(mod(ISTEP,ITRANS).eq.0)) then
            call TimerStart(T_MCTrans)
            call MCmodTrans(MCTransStepMType, RCUT, RECUT, RMAX, ALPHA, COULF, &
                BOX,RECvec, RKV,SSIN,SCOS,ChargeAtom,POTNB_riSjS,&
                NAtom,NMol,NA, NSiteMType,MTypeMol,AdrAtomMol,STypeAtom,&
                NKV, iPrint, stdout, LMoveMType,Ener, EEL,Svec,AccCntMType(:,:,2))
            call TimerStop(T_MCTrans)
        endif
        
    ! Molecule rotation MC step
        if ((IROT.gt.0).and.mod(ISTEP,IROT).eq.0) then
            call TimerStart(T_MCRot)
            call MCmodRot(NAtom,NA,NKV,NMol,IPRINT,stdout,NSiteMType,MTypeMol,AdrAtomMol,STypeAtom,SiteAtom,&
                BOX,MCRotStepMType,RCUT,RMAX,RECUT,COULF,ALPHA, ChargeAtom,MassSite,SumMassMType,PotNB_riSjS,LMoveMType,&
                Recvec,RKV,Svec,Wvec,XvecCOMMol,SSIN,SCOS,ENER,EEL,EINT,AccCntMType(:,:,3))
            call TimerStop(T_MCRot)
        endif ! End of rotational MC step       
        
    ! Averaging
        if((ISTEP.gt.NMCStepsEquil).and.(mod(ISTEP,IAV).eq.0)) then            
            call TimerStart(T_Average)
            call MCCorrel(NAtom,STypeAtom, NPointsPotB,IndexGlobB_riBond,IndexGlobNB_risjs,&  
                    RCUT,RMinPotB, RMaxPotB, LIMC,Bond_isjs,SiteAtom,CORS, CORP, NAV, npairsrdf)                    
            EINTS=EINTS+EINT
	    ENERS=ENERS+ENER
	    EELS=EELS+EEL
            call TimerStop(T_Average)
        end if ! end of Averaging
        
    ! Write coordinates to the trajectory file    
        if(ITR.gt.0.and.mod(ISTEP,ITR).eq.0)  call WriteTraj(psize,ISTEP,NAtom,NameSite,SiteAtom,Ri_vec,BOX)
                
    ! Write MC-simulation status to local output file
        if(mod(ISTEP,IOUT).eq.0) then 
            call TimerStart(T_reConstruct)
            ! Recalculate and Check possible deviations in the total energy. They can appear from errors in calculation of energy change during MC steps
            call MCmodConstruct(NA, RMAX, RCUT,RECUT, NAtom, NMol,BOX, COULF, ALPHA,&
                            Svec,Wvec,XvecCOMMol,STypeAtom, PotNB_riSjS, ChargeAtom, SiteAtom, MolAtom, IND,&
                            RMinPotB, RMaxPotB, NPointsPotB, PotB_riBond, AdrAtomMol,MTypeMol, MassSite,SumMassMType,&
                            iPrint,stdout, istep.gt.NMCStepsEquil, iStep)        
            call TimerStart(T_Ewald)
            call MCmodEwald(EFUR,Recvec,Svec,ChargeAtom,SSIN,SCOS,RKV,NAtomChrgAtom,AtomChrgAtom, NKV)                            
            call TimerStop(T_Ewald)
            call MCEnergy(ENER, EINT, EINT_ang, EEL, EFUR,ESF, iPrint, stdout)    
            ! Calculate virial and pressure
            VIR=sum(VIRij,mask=MASKNB)*0.5    
            VIRI=sum(VIRij,mask=MASKB)*0.5  
            VIRS=VIRS+VIR; VIIS=VIIS+VIRI; VIRE=VIRE-EEL
            PRES=(NAtom-(VIR+VIRij_frozen+VIRI+VIRIij_frozen-EEL)/3.)*FPR         
            ! Output thermodynamical variables
                        
            write(fmt,'(i3)') NMType*3            
            write((stdout),'(i9,a4,2f11.3,f10.3,f12.2,a4,'//trim(adjustl(fmt))//'f5.2)') ISTEP,' en:',&
                ENER*FKJM,EEL*FKJM,EINT*FKJM,&
                PRES,' a.r',&
                AccCntMType(:,1,1)*1./AccCntMType(:,2,1),&
                AccCntMType(:,1,2)*1./AccCntMType(:,2,2),&
                AccCntMType(:,1,3)*1./AccCntMType(:,2,3)
            if(IPRINT.ge.6) write((stdout),'(4f12.3)')-VIR*FPR/3.,EEL*FPR/3.,-VIRI*FPR/3,NAtom*FPR               
            if(IPRINT.ge.6) write((stdout),'(a,4f14.2)')' Pr.comp:',-VIR*FPR,-VIRI*FPR,-EFUR*FPR/3.,NAtom*FPR                
            call TimerStop(T_reConstruct)
            call flush(stdout)
        end if
                
    ! Perform local potential convergence check on every process (master and slaves)
        if (LpotCorrCheck.and.((ISTEP-NMCStepsEquil).gt.0).and.(mod(ISTEP-NMCStepsEquil,IPotCorrCheck).eq.0)) then
            write((stdout),*)"Local Potential correction convergence check",(ISTEP+1-NMCStepsEquil)/IPotCorrCheck," of ",&
                            (NMCSteps-NMCStepsEquil+1)/IPotCorrCheck
            ! Local potential correction (and intermediate RDF) on each process                
            call TimerStart(T_IMC)
            call POTCORRMain(CORS, REGP, RTM, DPOTM, CORP, RMIN, RMAX, VOL, FRT, PotB_riBond, PotNB_riSjS, RMinPotB, RMaxPotB,&
                IND, NPointsPotB, INDI, NMIN, NSType, NA, NBond, BTypeBond, MTypeBond, MTypeSType,&
                IndexGlobNB_riSjS, iSiteinBond, jSiteinBond, kSiteinBond, NPairsTotalBond, NSiteGroupsInBond, IndexGlobB_riBond,&
                LIMC, LRDF, BaseOutFileName,NameSite,NameSType,LEV,MixEV,npairsrdf,&
                NAV, .true., NMType, IREP, (ISTEP+1-int(NMCStepsEquil,4))/IPotCorrCheck,&
                StateNB_iSjS, StateB_iBond, ID, iPrint, stdout)
            call TimerStop(T_IMC)
#ifdef mpi
            if (ID.gt.0) then !If MPI - we gather all the data and perform correction check
                call TimerStart(T_MPI)
                call Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP,&
                npairsrdf,global_npairsrdf,LIMC, NMType)
                call TimerStop(T_MPI)
                if (ID.eq.1) then
! General potential correction and RDF
                    write(6,*) "Global Potential correction convergence check",(ISTEP+1-NMCStepsEquil)/IPotCorrCheck," of ",&
                                (NMCSteps-NMCStepsEquil+1)/IPotCorrCheck
                    call TimerStart(T_IMC)
                    call POTCORRMain(global_CORS,REGP,RTM,DPOTM,global_CORP,RMIN,RMAX,VOL,FRT,PotB_riBond,&
                        PotNB_riSjS,RMinPotB,RMaxPotB,&
                        IND,NPointsPotB,INDI,NMIN,NSType,NA,NBond,BTypeBond,MTypeBond,&
                        MTypeSType,&
                        IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NSiteGroupsInBond,&
                        IndexGlobB_riBond,LIMC,LRDF,BaseOutFileName,NameSite,NameSType,LEV,&
                        MixEV,global_npairsrdf,&
                        global_NAV,.true.,NMType,IREP,(ISTEP+1-int(NMCStepsEquil,4))/IPotCorrCheck, &
                        StateNB_iSjS, StateB_iBond,&
                        ID,iPrint,6) !Here I sent ID=0, to write output in general output file
                    call TimerStop(T_IMC)
                endif
            endif
#endif
        endif
        
        ! MCstep AutoAdjustment
        if (NMCAutoAdjust.gt.0) then
            if ((ISTEP.lt.NMCStepsEquil).and.(mod(ISTEP,NMCStepsEquil/NMCAutoAdjust).eq.0)) then
                call MCStepAutoAdjust(iStep,psize,iTrans, iRot,MCStepAtomARMType,MCTransStepARMType, MCRotStepARMType,&
                    MCStepAtomMType, MCTransStepMType, MCRotStepMType,AccCntMType,NameMType,ID,stdout)               
            endif
        endif
        
    enddo !==================== end of MC-simulation loop
    iStep=iStep-1 ! Make iStep=NMCSteps, to avoid any issues
!=======================
!   Write Monte Carlo simulation summary
    call PrintMCSummary(stdout,NAV,(1.d0*iOUT/NMCSteps),NMType,NMol,int(NMCSteps,kind=8),&
            NMCStepsEquil,NAtom,ITRANS,IROT,NMolMType,AccCntMType,BOX,MCStepAtomMType,MCTransStepMType,MCRotStepMType,&
            TEMP,EPS,EELS,EINTS,ENERS,VIRS,VIRE,VIIS,FPR,LRDF,NameMType,FILRDF,FILPOT)       

! Calculating resulting Local Potential Correcion (using data calculated just in this process)
    call TimerStart(T_IMC)            
    call POTCORRMain(CORS,REGP,RTM,DPOTM,CORP,RMIN,RMAX,&
            VOL,FRT,PotB_riBond,PotNB_riSjS,RMinPotB,RMaxPotB,IND,&
            NPointsPotB,INDI,NMIN,NSType,NA,NBond,BTypeBond,MTypeBond,&
            MTypeSType,IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NSiteGroupsInBond,&
            IndexGlobB_riBond,LIMC,LRDF, BaseOutFileName,NameSite,NameSType,&
            LEV,MixEV,npairsrdf,NAV,.false.,NMType,IREP,0,&
            StateNB_iSjS, StateB_iBond,&
            ID,iPrint, stdout)
    call TimerStop(T_IMC)
! If in parallel mode, gather data from all processes
            
#ifdef mpi
    if (ID.gt.0) then
        call TimerStart(T_MPI)
        call Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP, npairsrdf,global_npairsrdf,LIMC,NMType)
        ! Collecting and printing averaged calculation results:
        call CollectMCsummary(NAV,global_NAV,int(iStep,kind=8),global_MKSteps,AccCntMType,global_AccCntMType,EELS,&
            global_EELS,EINTS,global_EINTS,&
            ENERS,global_ENERS,VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE,LRDF,ID,psize,ierror)
        call TimerStop(T_MPI)
        
        if (ID.eq.1) then
            ! General resulting potential correction and RDF
            call PrintMCsummary(6,global_NAV,IOUT*1.d0/global_MKSteps,NMType,NMol,global_MKSteps,NMCStepsEquil*psize,&
                NAtom,ITRANS,IROT,NMolMType,global_AccCntMType,&
                BOX,MCStepAtomMType,MCTransStepMType,MCRotStepMType,TEMP,EPS,global_EELS,&
                global_EINTS,global_ENERS,global_VIRS,global_VIRE,global_VIIS,FPR, LRDF,NameMType,FILRDF,FILPOT)
            write(6,*)"Global Final Potential correction"
            call TimerStart(T_IMC)
            call POTCORRMain(global_CORS,REGP,RTM,DPOTM,global_CORP,RMIN,RMAX,VOL,FRT,PotB_riBond,&
                PotNB_riSjS,RMinPotB,RMaxPotB,IND,&
                NPointsPotB,INDI,NMIN,NSType,NA,NBond,BTypeBond,MTypeBond,&
                MTypeSType,&
                IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NSiteGroupsInBond,&
                IndexGlobB_riBond,LIMC,LRDF, BaseOutFileName,NameSite,NameSType,&
                LEV,MixEV,global_npairsrdf,&
                global_NAV,.false.,NMType,IREP,0,&
                StateNB_iSjS, StateB_iBond,&
                0,iPrint, 6) !Here I sent ID=0, to write output in general output file
            call TimerStop(T_IMC)
        endif
    endif
#endif
    ! Write the potential to a file: 
    call WritePot(BaseOutFileName,NactNB, NBond, NBondB, NBondA, NA, RMax, RMin, NSType, NameSType, NameMType,&
        StateNB_iSjS, PotNB_riSjS, MTypeBond, BTypeBond, NBondMType, AdrSiteMType, &
        NPointsPotB, RMaxPotB, RMinPotB, StateB_iBond, PotB_riBond,&
        NSiteGroupsInBond, iSiteinBond, jSiteinBond, kSiteinBond, FRT, irep)
    
    ! Write the final configuration of current IMC iteration to file
    if (LXMOL) call SaveLastConf(ID,NAtom,IREP,BaseOutFileName,NameSType,STypeAtom,Ri_vec,BOX, stdout)     
    ! Close the trajectory file for writing
    if (ITR.gt.0) close(19+psize)
    if(IREP.ge.IREPT) exit ! If no more inverse iterations to perform then exit.
    IREP=IREP+1
#ifdef mpi
    ! Passing new potential to all processes
    call TimerStart(T_MPI)
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)        
    call MPI_Bcast(PotNB_riSjS,size(PotNB_riSjS),MPI_REAL8,0,MPI_COMM_WORLD,ierror) !Sending POT        
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
    call MPI_Bcast(PotB_riBond,size(PotB_riBond),MPI_REAL8,0,MPI_COMM_WORLD,ierror) !Sending PotB_riBond
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
    call TimerStop(T_MPI)
#endif
enddo  ! end of IMC loop

! Deallocate arrays, close opened files, finalize MPI.
call TerminateMagiC(PotCorrJourn,SumMassMType,PotNB_riSjS,PotB_riBond,&
        XvecCOMMol,RSite,Svec, Wvec, MassSite, RMinPotB,RMaxPotB,&
        NSiteMType,AdrBondMType,AdrSiteMType,AdrMolMType,AdrAtomMType,AdrAtomMol,&
        NBondMType,Bond_iSjS,NPairsTotalBond,NSiteGroupsInBond, IndexGlobNB_riSjS,IndexGlobB_riBond,IND,INDI,MTypeSType,& 
        MTypeMol, MTypeSite,&
        iSiteinBond,jSiteinBond, kSiteinBond,NPointsPotB,NMIN, MolAtom,STypeAtom, SiteAtom, AtomChrgAtom,&
        CORS,CORP,npairsrdf,global_npairsrdf,global_CORP, global_CORS,& 
        NameSType, NameSite,ID,stdout,psize)
end
