!   This file contains subroutines responsible for output of MOLSIM
!       OpenTraj - Open/Create a file to write trajectory and connect to it. The file has name <BaseOutName>.i<iteration>.p<process>.traj.xmol
!       WriteTraj - Writes system's configuration to the trajectory file.
!       SaveLastConf - Writes resulting configuration of the system to a file. The file has name <BaseOutName>.i<iteration>.p<process>.start.xmol
!DEC$ FREEFORM

subroutine OpenTraj(ID,psize,IREP,BaseOutName)
    use precision
    !Global variables
    integer(4) ID, psize,IREP
    character(filename_text) BaseOutName 
    !Local variables
    character(filename_text) SaveFileName 
    
    SaveFileName=BaseOutName    
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.i'
    write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') IREP
    if (ID.ne.0) then
        SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.p'
        write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') ID
    endif
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+9)='.trj.xmol'
    open(unit=(19+psize),file=SaveFileName)    
end subroutine

subroutine WriteTraj(psize,ISTEP,NAtom,NameSite,SiteAtom,S, BOX)
    !Global variables
    use precision
    integer(4), intent(in)::  ISTEP,NAtom,psize, SiteAtom(:)
    real(8), intent(in) :: S(:,:), BOX(3)
    character(atom_name_len), intent(in) ::NameSite(:)
    ! Local variables
    integer(4) I
!    1.3  Dumping a configuration:
!  First Line - number of atoms
    write((19+psize),*) NAtom
!  Second commentary line of XMOL format contains time in fs or current MC-step
    write((19+psize),'(a6,i10,a,3f8.3)')' after ',ISTEP,' MC steps, BOX:', box        ! for MC
    do I=1,NAtom
        write((19+psize),'(a,3f14.5)')NameSite(SiteAtom(I)),S(1:3,I)
    end do   
end subroutine

subroutine SaveLastConf(ID,NAtom,IREP,OutFileName,NameSType,STypeAtom,Ri_vec,BOX, stdout)
! Global Variables
    use precision
    integer(4) IREP,STypeAtom(:),NAtom,ID, stdout
    real(8) Ri_vec(:,:)
    real(8) BOX(3)
    character(atom_name_len) NameSType(:)
    character(filename_text) OutFileName
! Local Variables
    character(filename_text) SaveFileName ! Local variable to work with, without changing global variable OutFileName
    integer(4) i
    SaveFileName=OutFileName
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.i'
    write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') IREP
    if (ID.ne.0) then
        SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.p'
        write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') ID
    endif
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+11)='.start.xmol'
    open(unit=2,file=SaveFileName,status='unknown')
    write(2,*)NAtom
    write(2,*)' start configuration BOX: ', BOX(1),' ', BOX(2),' ',BOX(3)
    do I=1,NAtom     ! all molecules
        write(2,'(a,3f14.5)')NameSType(STypeAtom(I)),Ri_vec(:,i)
    end do
    close(2)
    write((stdout),*)'Configuration snapshot file written ',SaveFilename
end subroutine