! Module for benchmarking of the code execution
!File : timingmod.f90
!Author : sasha
!
!Created on December 23, 2015, 2 : 07 PM
!

MODULE timer    
    
implicit none
! Named constants for accessing timing for different subroutines
integer(4), parameter:: T_N=11  ,& ! How many subroutines to benchmark
                        T_MCStep  = 1,  &
                        T_MCRot   = 2,  &
                        T_MCTrans = 3,  &
                        T_Average = 4,  &
                        T_reConstruct= 5,&
                        T_IMC     = 6,  &
                        T_Ewald   = 7,  &
                        T_DifEw   = 8,  &
                        T_MCStepdE= 9,  &
                        T_MPI     = 10, &
                        T_IMCinit = 11
real(8),private:: T_cpu(T_N), T_wall(T_N), T_wall_average(T_N), T_cpu_average(T_N)
real(8),private::  cnt_cpu_beg(T_N), cnt_cpu_end(T_N)
integer(8),private:: cnt_wall_beg(T_N), cnt_wall_end(T_N)
real(8), private :: t_wall_rate, t_cpu_rate, t_cpu_bigbang
integer(8), private:: cnt_rate,cnt_wall_bigbang
 
    
contains
subroutine TimerInit() ! Initiates the timer
    call system_clock(cnt_wall_bigbang, cnt_rate); t_wall_rate = real(cnt_rate)
    call cpu_time(t_cpu_bigbang)
end subroutine

subroutine TimerStart(ID) ! Start/resume timer for the specified routine
    integer(4),intent(in):: ID
    if ((ID.gt.T_N).or.(ID.le.0)) then
        write(*,*) 'Error: TimerStart: Wrong TaskID specified'
        stop
    endif
    call system_clock(cnt_wall_beg(ID))
    call cpu_time(cnt_cpu_beg(ID))
end subroutine

subroutine TimerStop(ID) !  Stop/pause timer for the specified routine
    integer(4),intent(in):: ID
    if ((ID.gt.T_N).or.(ID.le.0)) then
        write(*,*) 'Error: TimerStop: Wrong TaskID specified'
        stop
    endif
    call system_clock(cnt_wall_end(ID))
    call cpu_time(cnt_cpu_end(ID))
    T_wall(ID)=T_wall(ID)+(cnt_wall_end(ID)-cnt_wall_beg(ID))/t_wall_rate
    T_cpu(ID)=T_cpu(ID)+(cnt_cpu_end(ID)-cnt_cpu_beg(ID))
    
end subroutine

subroutine TimerCollectMPI(psize) ! Collect local processes timer data
integer(4), intent(in):: psize    
#ifdef mpi
integer(4) ierror
include 'mpif.h'
    call MPI_REDUCE(T_wall,T_wall_average,T_N,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(T_cpu,T_cpu_average,T_N,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
!    T_wall_average=T_wall_average/psize; T_cpu_average=T_cpu_average/psize
!    write(*,*) 'T_wall_average:',T_wall_average
!    write(*,*) 'T_cpu_average:',T_cpu_average
    T_wall=T_wall_average/psize; T_cpu=T_cpu_average/psize 
#endif    
end subroutine

subroutine TimerReport(stdout) ! Report timing measurements and statistics
    integer(4),intent(in):: stdout
    real(8) c
    integer(8) w 

    call system_clock(w); call cpu_time(c)

    write(stdout,*)               'Timing        T_wall(sec)   Tcpu(sec)'
    write(stdout,'(a15,2f12.2)')   'MagiC total  : ',(w-cnt_wall_bigbang)/t_wall_rate, c-t_cpu_bigbang
    write(stdout,'(a15,2f12.2,a)') 'MCStep       : ',T_wall(T_MCStep), T_cpu(T_MCStep),' including MCstepdE and DifEw'
    write(stdout,'(a15,2f12.2,a)') '  MCStep-excl: ',T_wall(T_MCStep)-T_wall(T_DifEw)-T_wall(T_MCStepdE),&
                                                    T_cpu(T_MCStep)-T_cpu(T_DifEw)-T_cpu(T_MCStepdE),' excluding MCstepdE and DifEw'
    write(stdout,'(a15,2f12.2)')   '  DifEw      : ',T_wall(T_DifEw), T_cpu(T_DifEw)
    write(stdout,'(a15,2f12.2)')   '  MCStepdE   : ',T_wall(T_MCStepdE), T_cpu(T_MCStepdE)
                                                    
    write(stdout,'(a15,2f12.2)')   'MCRot        : ',T_wall(T_MCRot), T_cpu(T_MCRot)
    write(stdout,'(a15,2f12.2)')   'MCTrans      : ',T_wall(T_MCTrans), T_cpu(T_MCTrans)
    write(stdout,'(a15,2f12.2)')   'Average      : ',T_wall(T_Average), T_cpu(T_Average)
    write(stdout,'(a15,2f12.2,a)') 'MCConstruct  : ',T_wall(T_reConstruct),T_cpu(T_reConstruct), ' including Ewald'    
    write(stdout,'(a15,2f12.2,a)') '  MCConstruct: ',T_wall(T_reConstruct)-T_wall(T_Ewald),&
                                                    T_cpu(T_reConstruct)-T_cpu(T_Ewald), ' excluding Ewald'
    write(stdout,'(a15,2f12.2)')   '  Ewald      : ',T_wall(T_Ewald), T_cpu(T_Ewald)                                                    
    write(stdout,'(a15,2f12.2)')   'IMC          : ',T_wall(T_IMC), T_cpu(T_IMC)
    write(stdout,'(a15,2f12.2)')   'IMC Init     : ',T_wall(T_IMCinit), T_cpu(T_IMCinit)    
    write(stdout,'(a15,2f12.2)')   'MPI          : ',T_wall(T_MPI), T_cpu(T_MPI)
end subroutine

subroutine TimerStamp(stdout,text)
    character(*), intent(in):: text
    integer(4),intent(in):: stdout
    real(8) c
    integer(8) w 

    call system_clock(w); call cpu_time(c)
    write(stdout,*) text,' WallTime(sec): ',(w-cnt_wall_bigbang)/t_wall_rate, ' CPU Time(sec): ',c-t_cpu_bigbang     
end subroutine
       
END
